@@ -1,4 +1,4 @@
-# Copyright (C) 2010  Kouhei Sutou <kou@clear-code.com>
+# Copyright (C) 2010-2011  Kouhei Sutou <kou@clear-code.com>
 #
 # This library is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Lesser General Public License as published by
@@ -51,22 +51,28 @@ module Milter
         if @options.unix_socket_mode
           client.unix_socket_mode = @options.unix_socket_mode
         end
+        client.event_loop_backend = @options.event_loop_backend
+        client.default_packet_buffer_size = @options.packet_buffer_size
+        client.n_workers = @options.n_workers
         yield(client, options) if block_given?
         daemonize if @options.run_as_daemon
-        client.main
+        client.run
       end
 
       private
       def setup_options
         @options = OpenStruct.new
         @options.connection_spec = "inet:20025"
-	@options.status_on_error = "accept"
+        @options.status_on_error = "accept"
         @options.run_as_daemon = false
         @options.user = nil
         @options.group = nil
         @options.unix_socket_group = nil
         @options.unix_socket_mode = nil
         @options.syslog = false
+        @options.event_loop_backend = Milter::Client::EVENT_LOOP_BACKEND_GLIB
+        @options.n_workers = 0
+        @options.packet_buffer_size = 0
       end
 
       def setup_option_parser
@@ -131,23 +137,53 @@ module Milter
 
         @option_parser.on("--group=GROUP",
                           "Run as GROUP's process (need root privilege)",
-                          "(#{@options.group})") do |gorup|
+                          "(#{@options.group})") do |group|
           @options.group = group
         end
 
         @option_parser.on("--unix-socket-group=GROUP",
                           "Change UNIX domain socket group to GROUP",
-                          "(#{@options.unix_socket_group})") do |gorup|
+                          "(#{@options.unix_socket_group})") do |group|
           @options.unix_socket_group = group
         end
 
         client = Milter::Client.new
         @option_parser.on("--unix-socket-mode=MODE",
                           "Change UNIX domain socket mode to MODE",
-                          "(#{client.default_unix_socket_mode})") do |mode|
+                          "(%o)" % client.default_unix_socket_mode) do |mode|
           client.unix_socket_mode = mode
           @options.unix_socket_mode = mode
         end
+
+        backends = Milter::ClientEventLoopBackend.values.collect do |value|
+          value.nick
+        end
+        @option_parser.on("--event-loop-backend=BACKEND", backends,
+                          "Use BACKEND as event loop backend.",
+                          "available values: [#{backends.join(', ')}]",
+                          "(#{@options.event_loop_backend.nick})") do |backend|
+          @options.event_loop_backend = backend
+        end
+
+        @option_parser.on("--n-workers=N",
+                          Integer,
+                          "Run with N workrs.",
+                          "(#{@options.n_workers})") do |num|
+          if num <= 0
+            raise OptionParser::InvalidArgument
+          end
+          @options.n_workers = num
+        end
+
+        @option_parser.on("--packet-buffer-size=SIZE",
+                          Integer,
+                          "Use SIZE as packet buffer size.",
+                          "(#{@options.packet_buffer_size})") do |size|
+          if size < 0
+            raise OptionParser::InvalidArgument
+          end
+          @options.packet_buffer_size = size
+        end
       end
 
       def setup_logger_options
@@ -183,6 +219,7 @@ module Milter
       def daemonize
         WEBrick::Daemon.start
       end
+
     end
   end
 end
