@@ -1,6 +1,6 @@
 # Author:: Nathaniel Talbott.
 # Copyright:: Copyright (c) 2000-2003 Nathaniel Talbott. All rights reserved.
-#             Copyright (c) 2009 Kouhei Sutou.
+#             Copyright (c) 2009-2010 Kouhei Sutou. All rights reserved.
 # License:: Ruby license.
 
 require 'test/unit/assertionfailederror'
@@ -80,10 +80,23 @@ module Test
       public
       def assert_equal(expected, actual, message=nil)
         diff = AssertionMessage.delayed_diff(expected, actual)
-        full_message = build_message(message, <<EOT, expected, actual, diff)
+        if expected.respond_to?(:encoding) and
+            actual.respond_to?(:encoding) and
+            expected.encoding != actual.encoding
+          format = <<EOT
+<?>(?) expected but was
+<?>(?).?
+EOT
+          full_message = build_message(message, format,
+                                       expected, expected.encoding.name,
+                                       actual, actual.encoding.name,
+                                       diff)
+        else
+          full_message = build_message(message, <<EOT, expected, actual, diff)
 <?> expected but was
 <?>.?
 EOT
+        end
         begin
           assert_block(full_message) { expected == actual }
         rescue AssertionFailedError => failure
@@ -539,19 +552,79 @@ EOT
       public
       def assert_in_delta(expected_float, actual_float, delta, message="")
         _wrap_assertion do
-          {expected_float => "first float", actual_float => "second float", delta => "delta"}.each do |float, name|
-            assert_respond_to(float, :to_f, "The arguments must respond to to_f; the #{name} did not")
+          _assert_in_delta_validate_arguments(expected_float,
+                                              actual_float,
+                                              delta)
+          full_message = _assert_in_delta_message(expected_float,
+                                                  actual_float,
+                                                  delta,
+                                                  message)
+          assert_block(full_message) do
+            (expected_float.to_f - actual_float.to_f).abs <= delta.to_f
           end
-          assert_operator(delta, :>=, 0.0, "The delta should not be negative")
-          full_message = build_message(message, <<EOT, expected_float, actual_float, delta)
-<?> and
-<?> expected to be within
-<?> of each other.
-EOT
-          assert_block(full_message) { (expected_float.to_f - actual_float.to_f).abs <= delta.to_f }
         end
       end
 
+      # :stopdoc:
+      private
+      def _assert_in_delta_validate_arguments(expected_float,
+                                              actual_float,
+                                              delta)
+        {
+          expected_float => "first float",
+          actual_float => "second float",
+          delta => "delta"
+        }.each do |float, name|
+          assert_respond_to(float, :to_f,
+                            "The arguments must respond to to_f; " +
+                            "the #{name} did not")
+        end
+        delta = delta.to_f
+        assert_operator(delta, :>=, 0.0, "The delta should not be negative")
+      end
+
+      def _assert_in_delta_message(expected_float, actual_float, delta,
+                                   message)
+        format = <<-EOT
+<?> expected but was
+<?> (tolerance <?>).
+EOT
+        arguments = [expected_float, actual_float, delta]
+        normalized_expected = expected_float.to_f
+        normalized_actual = actual_float.to_f
+        normalized_delta = delta.to_f
+        relation_format = nil
+        relation_arguments = nil
+        if normalized_actual < normalized_expected - normalized_delta
+          relation_format = "<<?> < <?>-<?>(?) <= <?>+<?>(?)>"
+          relation_arguments = [actual_float,
+                                expected_float, delta,
+                                normalized_expected - normalized_delta,
+                                expected_float, delta,
+                                normalized_expected + normalized_delta]
+        elsif normalized_expected - normalized_delta < normalized_actual
+          relation_format = "<<?>-<?>(?) <= <?>+<?>(?) < <?>>"
+          relation_arguments = [expected_float, delta,
+                                normalized_expected - normalized_delta,
+                                expected_float, delta,
+                                normalized_expected + normalized_delta,
+                                actual_float]
+        end
+
+        if relation_format
+          format << <<-EOT
+
+Relation:
+#{relation_format}
+EOT
+          arguments.concat(relation_arguments)
+        end
+
+        build_message(message, format, *arguments)
+      end
+      public
+      # :startdoc:
+
       ##
       # Passes if the method send returns a true value.
       #
@@ -846,6 +919,42 @@ EOT
         end
       end
 
+      ##
+      # Passes if +path+ exists.
+      #
+      # Example:
+      #   assert_path_exist("/tmp")          # -> pass
+      #   assert_path_exist("/bin/sh")       # -> pass
+      #   assert_path_exist("/nonexistent")  # -> fail
+      def assert_path_exist(path, message=nil)
+        _wrap_assertion do
+          failure_message = build_message(message,
+                                          "<?> expected to exist",
+                                          path)
+          assert_block(failure_message) do
+            File.exist?(path)
+          end
+        end
+      end
+
+      ##
+      # Passes if +path+ doesn't exist.
+      #
+      # Example:
+      #   assert_path_not_exist("/nonexistent")  # -> pass
+      #   assert_path_not_exist("/tmp")          # -> fail
+      #   assert_path_not_exist("/bin/sh")       # -> fail
+      def assert_path_not_exist(path, message=nil)
+        _wrap_assertion do
+          failure_message = build_message(message,
+                                          "<?> expected to not exist",
+                                          path)
+          assert_block(failure_message) do
+            not File.exist?(path)
+          end
+        end
+      end
+
       ##
       # Builds a failure message.  +head+ is added before the +template+ and
       # +arguments+ replaces the '?'s positionally in the template.
