@@ -7,6 +7,7 @@ module Test
       RUNNERS = {}
       COLLECTORS = {}
       ADDITIONAL_OPTIONS = []
+      PREPARE_HOOKS = []
 
       class << self
         def register_runner(id, runner_builder=Proc.new)
@@ -43,15 +44,20 @@ module Test
         def setup_option(option_builder=Proc.new)
           ADDITIONAL_OPTIONS << option_builder
         end
+
+        def prepare(hook=Proc.new)
+          PREPARE_HOOKS << hook
+        end
       end
 
       def self.run(force_standalone=false, default_dir=nil, argv=ARGV, &block)
         r = new(force_standalone || standalone?, &block)
         r.base = default_dir
+        r.prepare
         r.process_args(argv)
         r.run
       end
-      
+
       def self.standalone?
         return false unless("-e" == $0)
         ObjectSpace.each_object(Class) do |klass|
@@ -99,7 +105,7 @@ module Test
 
       attr_reader :suite, :runner_options
       attr_accessor :filters, :to_run, :pattern, :exclude, :base, :workdir
-      attr_accessor :color_scheme
+      attr_accessor :color_scheme, :listeners
       attr_writer :runner, :collector
 
       def initialize(standalone)
@@ -113,6 +119,7 @@ module Test
         @runner_options = {}
         @default_arguments = []
         @workdir = nil
+        @listeners = []
         config_file = "test-unit.yml"
         if File.exist?(config_file)
           load_config(config_file)
@@ -122,6 +129,12 @@ module Test
         yield(self) if block_given?
       end
 
+      def prepare
+        PREPARE_HOOKS.each do |handler|
+          handler.call(self)
+        end
+      end
+
       def process_args(args = ARGV)
         begin
           args.unshift(*@default_arguments)
@@ -300,6 +313,8 @@ module Test
         runner = @runner[self]
         return false if runner.nil?
         @runner_options[:color_scheme] ||= @color_scheme
+        @runner_options[:listeners] ||= []
+        @runner_options[:listeners].concat(@listeners)
         Dir.chdir(@workdir) if @workdir
         runner.run(suite, @runner_options).passed?
       end
@@ -342,7 +357,7 @@ module Test
       end
 
       def global_config_file
-        File.expand_path("~/.test-unit.xml")
+        File.expand_path("~/.test-unit.yml")
       rescue ArgumentError
         nil
       end
