@@ -1,4 +1,4 @@
-# Copyright (C) 2008-2010  Kouhei Sutou <kou@clear-code.com>
+# Copyright (C) 2008-2011  Kouhei Sutou <kou@clear-code.com>
 #
 # This library is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Lesser General Public License as published by
@@ -14,8 +14,11 @@
 # along with this library.  If not, see <http://www.gnu.org/licenses/>.
 
 class TestConfiguration < Test::Unit::TestCase
+  include MilterEventLoopTestUtils
+
   def setup
     @configuration = Milter::Manager::Configuration.new
+    @loop = create_event_loop
   end
 
   def test_controller_connection_spec
@@ -63,6 +66,26 @@ class TestConfiguration < Test::Unit::TestCase
     assert_equal(10, @configuration.connection_check_interval)
   end
 
+  def test_event_loop_backend
+    assert_equal("glib", @configuration.event_loop_backend.nick)
+    @configuration.event_loop_backend = "libev"
+    assert_equal("libev", @configuration.event_loop_backend.nick)
+    @configuration.event_loop_backend = "glib"
+    assert_equal("glib", @configuration.event_loop_backend.nick)
+  end
+
+  def test_n_workers
+    assert_equal(0, @configuration.n_workers)
+    @configuration.n_workers = 10
+    assert_equal(10, @configuration.n_workers)
+  end
+
+  def test_default_packet_buffer_size
+    assert_equal(0, @configuration.default_packet_buffer_size)
+    @configuration.default_packet_buffer_size = 4096
+    assert_equal(4096, @configuration.default_packet_buffer_size)
+  end
+
   def test_package
     @configuration.package_platform = "pkgsrc"
     assert_equal("pkgsrc", @configuration.package_platform)
@@ -250,6 +273,16 @@ manager.max_file_descriptors = 0
 # default
 manager.custom_configuration_directory = nil
 # default
+manager.fallback_status = "accept"
+# default
+manager.fallback_status_at_disconnect = "temporary-failure"
+# default
+manager.event_loop_backend = "glib"
+# default
+manager.n_workers = 0
+# default
+manager.packet_buffer_size = 0
+# default
 manager.connection_check_interval = 0
 
 # default
@@ -352,6 +385,16 @@ manager.max_file_descriptors = 0
 # #{__FILE__}:#{manager_custom_configuration_directory}
 manager.custom_configuration_directory = "/tmp/milter-manager/"
 # default
+manager.fallback_status = "accept"
+# default
+manager.fallback_status_at_disconnect = "temporary-failure"
+# default
+manager.event_loop_backend = "glib"
+# default
+manager.n_workers = 0
+# default
+manager.packet_buffer_size = 0
+# default
 manager.connection_check_interval = 0
 
 # #{__FILE__}:#{controller_connection_spec}
@@ -459,7 +502,7 @@ EOD
 
   private
   def create_children
-    children = Milter::Manager::Children.new(@configuration)
+    children = Milter::Manager::Children.new(@configuration, @loop)
     client_context = Milter::ClientContext.new
     @configuration.setup_children(children, client_context)
     children
