@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2008-2009  Kouhei Sutou <kou@clear-code.com>
+# Copyright (C) 2008-2010  Kouhei Sutou <kou@clear-code.com>
 #
 # This library is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Lesser General Public License as published by
@@ -88,8 +88,8 @@ class TestDebianInitDetector < Test::Unit::TestCase
     detector.detect
     assert_equal({
                    "Provides" => "milter-manager",
-                   "Required-Start" => "$local_fs $remote_fs $network $syslog",
-                   "Required-Stop" => "$local_fs $remote_fs $network $syslog",
+                   "Required-Start" => "$remote_fs $network $syslog",
+                   "Required-Stop" => "$remote_fs $network $syslog",
                    "Default-Start" => "2 3 4 5",
                    "Default-Stop" => "0 1 6",
                    "Short-Description" => "milter-manager's init script",
@@ -573,11 +573,12 @@ EOC
   end
 
   private
-  def debain_init_detector(name)
+  def debain_init_detector(name, options={})
     detector = Milter::Manager::DebianInitDetector.new(@configuration, name)
 
     _init_base_dir = @init_base_dir
     _default_dir = @default_dir
+    _candidate_service_commands = options[:candidate_service_commands] || []
     singleton_object = class << detector; self; end
     singleton_object.send(:define_method, :init_base_dir) do
       _init_base_dir.to_s
@@ -587,6 +588,10 @@ EOC
       _default_dir.to_s
     end
 
+    singleton_object.send(:define_method, :candidate_service_commands) do
+      _candidate_service_commands
+    end
+
     detector
   end
 
