@@ -27,6 +27,9 @@
 #include "milter-message-result.h"
 #include "milter-session-result.h"
 #include "milter-memory-profile.h"
+#include "milter-event-loop.h"
+#include "milter-libev-event-loop.h"
+#include "milter-glib-event-loop.h"
 
 /* enumerations from "milter-protocol.h" */
 GType
@@ -217,6 +220,7 @@ milter_agent_error_get_type (void)
     static const GEnumValue values[] = {
       { MILTER_AGENT_ERROR_IO_ERROR, "MILTER_AGENT_ERROR_IO_ERROR", "io-error" },
       { MILTER_AGENT_ERROR_DECODE_ERROR, "MILTER_AGENT_ERROR_DECODE_ERROR", "decode-error" },
+      { MILTER_AGENT_ERROR_NO_EVENT_LOOP_ERROR, "MILTER_AGENT_ERROR_NO_EVENT_LOOP_ERROR", "no-event-loop-error" },
       { 0, NULL, NULL }
     };
     etype = g_enum_register_static ("MilterAgentError", values);
@@ -342,6 +346,8 @@ milter_log_item_flags_get_type (void)
       { MILTER_LOG_ITEM_LEVEL, "MILTER_LOG_ITEM_LEVEL", "level" },
       { MILTER_LOG_ITEM_LOCATION, "MILTER_LOG_ITEM_LOCATION", "location" },
       { MILTER_LOG_ITEM_TIME, "MILTER_LOG_ITEM_TIME", "time" },
+      { MILTER_LOG_ITEM_NAME, "MILTER_LOG_ITEM_NAME", "name" },
+      { MILTER_LOG_ITEM_PID, "MILTER_LOG_ITEM_PID", "pid" },
       { 0, NULL, NULL }
     };
     etype = g_flags_register_static ("MilterLogItemFlags", values);
@@ -364,6 +370,20 @@ milter_log_colorize_get_type (void)
   return etype;
 }
 GType
+milter_utils_pipe_mode_get_type (void)
+{
+  static GType etype = 0;
+  if (etype == 0) {
+    static const GEnumValue values[] = {
+      { MILTER_UTILS_READ_PIPE, "MILTER_UTILS_READ_PIPE", "read-pipe" },
+      { MILTER_UTILS_WRITE_PIPE, "MILTER_UTILS_WRITE_PIPE", "write-pipe" },
+      { 0, NULL, NULL }
+    };
+    etype = g_enum_register_static ("MilterUtilsPipeMode", values);
+  }
+  return etype;
+}
+GType
 milter_connection_error_get_type (void)
 {
   static GType etype = 0;
@@ -436,6 +456,19 @@ milter_state_get_type (void)
   }
   return etype;
 }
+GType
+milter_event_loop_error_get_type (void)
+{
+  static GType etype = 0;
+  if (etype == 0) {
+    static const GEnumValue values[] = {
+      { MILTER_EVENT_LOOP_ERROR_MAX, "MILTER_EVENT_LOOP_ERROR_MAX", "max" },
+      { 0, NULL, NULL }
+    };
+    etype = g_enum_register_static ("MilterEventLoopError", values);
+  }
+  return etype;
+}
 
 /* Generated data ends here */
 
