@@ -42,12 +42,16 @@ GType milter_log_item_flags_get_type (void);
 #define MILTER_TYPE_LOG_ITEM_FLAGS (milter_log_item_flags_get_type())
 GType milter_log_colorize_get_type (void);
 #define MILTER_TYPE_LOG_COLORIZE (milter_log_colorize_get_type())
+GType milter_utils_pipe_mode_get_type (void);
+#define MILTER_TYPE_UTILS_PIPE_MODE (milter_utils_pipe_mode_get_type())
 GType milter_connection_error_get_type (void);
 #define MILTER_TYPE_CONNECTION_ERROR (milter_connection_error_get_type())
 GType milter_esmtp_error_get_type (void);
 #define MILTER_TYPE_ESMTP_ERROR (milter_esmtp_error_get_type())
 GType milter_state_get_type (void);
 #define MILTER_TYPE_STATE (milter_state_get_type())
+GType milter_event_loop_error_get_type (void);
+#define MILTER_TYPE_EVENT_LOOP_ERROR (milter_event_loop_error_get_type())
 G_END_DECLS
 
 #endif /* __MILTER_ENUM_TYPES_H__ */
