@@ -1,6 +1,6 @@
 /* -*- Mode: C; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /*
- *  Copyright (C) 2008-2010  Kouhei Sutou <kou@clear-code.com>
+ *  Copyright (C) 2008-2011  Kouhei Sutou <kou@clear-code.com>
  *
  *  This library is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -204,6 +204,11 @@ MilterStatus  milter_manager_configuration_get_fallback_status
 void          milter_manager_configuration_set_fallback_status
                                      (MilterManagerConfiguration *configuration,
                                       MilterStatus                status);
+MilterStatus  milter_manager_configuration_get_fallback_status_at_disconnect
+                                     (MilterManagerConfiguration *configuration);
+void          milter_manager_configuration_set_fallback_status_at_disconnect
+                                     (MilterManagerConfiguration *configuration,
+                                      MilterStatus                status);
 
 const gchar  *milter_manager_configuration_get_package_platform
                                      (MilterManagerConfiguration *configuration);
@@ -328,6 +333,24 @@ guint         milter_manager_configuration_get_connection_check_interval
 void          milter_manager_configuration_set_connection_check_interval
                                      (MilterManagerConfiguration *configuration,
                                       guint                       interval_in_seconds);
+MilterClientEventLoopBackend
+              milter_manager_configuration_get_event_loop_backend
+                                     (MilterManagerConfiguration *configuration);
+void          milter_manager_configuration_set_event_loop_backend
+                                     (MilterManagerConfiguration      *configuration,
+                                      MilterClientEventLoopBackend     backend);
+
+guint         milter_manager_configuration_get_n_workers
+                                     (MilterManagerConfiguration *configuration);
+void          milter_manager_configuration_set_n_workers
+                                     (MilterManagerConfiguration *configuration,
+                                      guint                       n_workers);
+
+guint         milter_manager_configuration_get_default_packet_buffer_size
+                                     (MilterManagerConfiguration *configuration);
+void          milter_manager_configuration_set_default_packet_buffer_size
+                                     (MilterManagerConfiguration *configuration,
+                                      guint                       size);
 
 G_END_DECLS
 
