@@ -1,6 +1,6 @@
 /* -*- Mode: C; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /*
- *  Copyright (C) 2008-2010  Kouhei Sutou <kou@clear-code.com>
+ *  Copyright (C) 2008-2011  Kouhei Sutou <kou@clear-code.com>
  *
  *  This library is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -271,10 +271,16 @@ cb_idle_reload_configuration (gpointer user_data)
 static void
 reload_configuration_request (int signum)
 {
-    g_idle_add_full(G_PRIORITY_DEFAULT,
-                    cb_idle_reload_configuration,
-                    NULL,
-                    NULL);
+    if (the_manager) {
+        MilterEventLoop *loop;
+
+        loop = milter_agent_get_event_loop(MILTER_AGENT(the_manager));
+        milter_event_loop_add_idle_full(loop,
+                                        G_PRIORITY_DEFAULT,
+                                        cb_idle_reload_configuration,
+                                        NULL,
+                                        NULL);
+    }
 }
 
 static void
@@ -283,13 +289,8 @@ cb_error (MilterErrorEmittable *emittable, GError *error, gpointer user_data)
     g_print("milter-manager error: %s\n", error->message);
 }
 
-typedef enum {
-    READ_PIPE,
-    WRITE_PIPE
-} PipeMode;
-
 static void
-close_pipe (int *pipe, PipeMode mode)
+close_pipe (int *pipe, MilterUtilsPipeMode mode)
 {
     if (pipe[mode] == -1)
         return;
@@ -334,8 +335,8 @@ create_process_launcher_pipes (gint **command_pipe, gint **reply_pipe)
     if (pipe(*reply_pipe) == -1) {
         milter_manager_error("failed to create pipe for launcher reply: %s",
                              g_strerror(errno));
-        close((*command_pipe)[WRITE_PIPE]);
-        close((*command_pipe)[READ_PIPE]);
+        close((*command_pipe)[MILTER_UTILS_WRITE_PIPE]);
+        close((*command_pipe)[MILTER_UTILS_READ_PIPE]);
         return FALSE;
     }
 
@@ -348,11 +349,11 @@ prepare_process_launcher_pipes_for_process_launcher (gint *command_pipe,
                                                      GIOChannel **read_channel,
                                                      GIOChannel **write_channel)
 {
-    close_pipe(command_pipe, WRITE_PIPE);
-    close_pipe(reply_pipe, READ_PIPE);
+    close_pipe(command_pipe, MILTER_UTILS_WRITE_PIPE);
+    close_pipe(reply_pipe, MILTER_UTILS_READ_PIPE);
 
-    *write_channel = create_write_io_channel(reply_pipe[WRITE_PIPE]);
-    *read_channel = create_read_io_channel(command_pipe[READ_PIPE]);
+    *write_channel = create_write_io_channel(reply_pipe[MILTER_UTILS_WRITE_PIPE]);
+    *read_channel = create_read_io_channel(command_pipe[MILTER_UTILS_READ_PIPE]);
 }
 
 static void
@@ -361,22 +362,25 @@ prepare_process_launcher_pipes_for_manager (gint *command_pipe,
                                             GIOChannel **read_channel,
                                             GIOChannel **write_channel)
 {
-    close_pipe(command_pipe, READ_PIPE);
-    close_pipe(reply_pipe, WRITE_PIPE);
+    close_pipe(command_pipe, MILTER_UTILS_READ_PIPE);
+    close_pipe(reply_pipe, MILTER_UTILS_WRITE_PIPE);
 
-    *write_channel = create_write_io_channel(command_pipe[WRITE_PIPE]);
-    *read_channel = create_read_io_channel(reply_pipe[READ_PIPE]);
+    *write_channel = create_write_io_channel(command_pipe[MILTER_UTILS_WRITE_PIPE]);
+    *read_channel = create_read_io_channel(reply_pipe[MILTER_UTILS_READ_PIPE]);
 }
 
-static void
+static gboolean
 start_process_launcher (GIOChannel *read_channel, GIOChannel *write_channel,
                         gboolean daemon)
 {
+    gboolean success = TRUE;
     MilterManagerProcessLauncher *launcher;
     MilterReader *reader;
     MilterWriter *writer;
     gchar *error_message = NULL;
     MilterSyslogLogger *logger;
+    MilterEventLoop *loop;
+    GError *error = NULL;
 
     logger = milter_syslog_logger_new("milter-manager-process-launcher");
 
@@ -391,19 +395,32 @@ start_process_launcher (GIOChannel *read_channel, GIOChannel *write_channel,
     g_object_unref(reader);
     g_object_unref(writer);
 
-    milter_agent_start(MILTER_AGENT(launcher), NULL);
+    loop = milter_glib_event_loop_new(NULL);
+    milter_agent_set_event_loop(MILTER_AGENT(launcher), loop);
+    g_object_unref(loop);
 
-    if (daemon && !milter_utils_detach_io(&error_message)) {
+    if (!milter_agent_start(MILTER_AGENT(launcher), &error)) {
+        milter_error("[process-launcher][error][start] %s",
+                     error->message);
+        g_error_free(error);
+        success = FALSE;
+    }
+
+    if (success && daemon && !milter_utils_detach_io(&error_message)) {
         milter_error("[process-launcher][error] failed to detach IO: %s",
                      error_message);
         g_free(error_message);
+        success = FALSE;
     }
 
-    milter_manager_process_launcher_run(launcher);
+    if (success)
+        milter_manager_process_launcher_run(launcher);
 
     g_object_unref(launcher);
 
     g_object_unref(logger);
+
+    return success;
 }
 
 static gboolean
@@ -423,7 +440,14 @@ start_process_launcher_process (MilterManager *manager)
     if (!create_process_launcher_pipes(&command_pipe_p, &reply_pipe_p))
         return FALSE;
 
-    switch (fork()) {
+    milter_debug("[manager][process-launcher][pipe] "
+                 "command:<%d:%d>, reply:<%d:%d>",
+                 command_pipe[MILTER_UTILS_READ_PIPE],
+                 command_pipe[MILTER_UTILS_WRITE_PIPE],
+                 reply_pipe[MILTER_UTILS_READ_PIPE],
+                 reply_pipe[MILTER_UTILS_WRITE_PIPE]);
+
+    switch (milter_client_fork(MILTER_CLIENT(manager))) {
     case 0:
         /* FIXME: change process name to
            'milter-manager: process-launcher' by
@@ -436,8 +460,11 @@ start_process_launcher_process (MilterManager *manager)
                                                             reply_pipe_p,
                                                             &read_channel,
                                                             &write_channel);
-        start_process_launcher(read_channel, write_channel, daemon);
-        _exit(EXIT_FAILURE);
+        if (start_process_launcher(read_channel, write_channel, daemon)) {
+            _exit(EXIT_SUCCESS);
+        } else {
+            _exit(EXIT_FAILURE);
+        }
         break;
     case -1:
         milter_manager_error("failed to fork process launcher process: %s",
@@ -656,6 +683,7 @@ milter_manager_main (void)
     MilterManager *manager;
     MilterManagerController *controller;
     MilterManagerConfiguration *config;
+    MilterEventLoop *loop;
     gboolean remove_socket, daemon;
     gchar *pid_file = NULL;
     GError *error = NULL;
@@ -719,7 +747,8 @@ milter_manager_main (void)
         return FALSE;
     }
 
-    controller = milter_manager_controller_new(manager);
+    loop = milter_client_get_process_loop(client);
+    controller = milter_manager_controller_new(manager, loop);
     if (controller && !milter_manager_controller_listen(controller, &error)) {
         milter_manager_error("failed to listen controller socket: %s",
                              error->message);
@@ -782,8 +811,11 @@ milter_manager_main (void)
     SET_SIGNAL_ACTION(HUP, hup, reload_configuration_request_action);
 #undef SET_SIGNAL_ACTION
 
-    if (!milter_client_main(client))
-        milter_manager_error("failed to start milter-manager process.");
+    if (!milter_client_run(client, &error)) {
+        milter_manager_error("failed to start milter-manager process: %s",
+                             error->message);
+        g_error_free(error);
+    }
 
 #define UNSET_SIGNAL_ACTION(SIGNAL, signal)                             \
     if (set_sig ## signal ## _action)                                   \
