@@ -13,11 +13,10 @@
  *  GNU Lesser General Public License for more details.
  *
  *  You should have received a copy of the GNU Lesser General Public License
- *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *  along with this library.  If not, see <http://www.gnu.org/licenses/>.
  *
  */
 
-
 #include <rb-milter-manager.h>
 
 #undef PACKAGE_NAME
@@ -26,21 +25,21 @@
 #undef PACKAGE_STRING
 #undef PACKAGE_BUGREPORT
 
-#ifndef HAVE_RB_ERRINFO
-#  define rb_errinfo() (ruby_errinfo)
-#endif
-
-
 #ifdef HAVE_CONFIG_H
-#  include <config.h>
+#  include "../../../config.h"
 #endif /* HAVE_CONFIG_H */
 
+
 #include <signal.h>
 #include <gmodule.h>
 
 #include <milter/manager/milter-manager-configuration.h>
 #include <milter/manager/milter-manager-module-impl.h>
 
+#ifndef HAVE_RB_ERRINFO
+#  define rb_errinfo() (ruby_errinfo)
+#endif
+
 #define MILTER_TYPE_MANAGER_RUBY_CONFIGURATION            (milter_manager_ruby_configuration_get_type())
 #define MILTER_MANAGER_RUBY_CONFIGURATION(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), MILTER_TYPE_MANAGER_RUBY_CONFIGURATION, MilterManagerRubyConfiguration))
 #define MILTER_MANAGER_RUBY_CONFIGURATION_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), MILTER_TYPE_MANAGER_RUBY_CONFIGURATION, MilterManagerRubyConfigurationClass))
@@ -53,6 +52,7 @@ typedef struct _MilterManagerRubyConfigurationClass MilterManagerRubyConfigurati
 struct _MilterManagerRubyConfiguration
 {
     MilterManagerConfiguration object;
+    gboolean disposing;
 };
 
 struct _MilterManagerRubyConfigurationClass
@@ -121,6 +121,7 @@ milter_manager_ruby_configuration_class_finalize (MilterManagerRubyConfiguration
 static void
 milter_manager_ruby_configuration_init (MilterManagerRubyConfiguration *configuration)
 {
+    configuration->disposing = FALSE;
 }
 
 static void
@@ -244,6 +245,7 @@ rb_funcall_protect (GError **g_error, VALUE receiver, ID name, guint argc, ...)
                         (VALUE)&arguments_data,
                         &state);
     va_end(args);
+    xfree(argv);
 
     error = rb_errinfo();
     if (state && !NIL_P(error)) {
@@ -257,9 +259,9 @@ rb_funcall_protect (GError **g_error, VALUE receiver, ID name, guint argc, ...)
         g_string_append_printf(error_message, "%s (%s)\n",
                                RVAL2CSTR(message), RVAL2CSTR(class_name));
         backtrace = rb_funcall(error, rb_intern("backtrace"), 0);
-        for (i = 0; i < RARRAY(backtrace)->len; i++) {
+        for (i = 0; i < RARRAY_LEN(backtrace); i++) {
             g_string_append_printf(error_message, "%s\n",
-                                   RVAL2CSTR(RARRAY(backtrace)->ptr[i]));
+                                   RVAL2CSTR(RARRAY_PTR(backtrace)[i]));
         }
         g_set_error(g_error,
                     MILTER_MANAGER_CONFIGURATION_ERROR,
@@ -315,21 +317,26 @@ static void
 init_ruby (void)
 {
     int argc;
-    char *argv[] = {"milter-manager"};
+    static char args[sizeof "milter-manager\0-e;\0"];
+    char *argv[3], *arg;
     const gchar *milter_manager_rubylib;
 
-    argc = sizeof(argv) / sizeof(*argv);
+    argc = 0;
+    arg = args;
+    argv[argc++] = arg;
+    arg += strlen(strcpy(arg, "milter-manager")) + 1;
+    argv[argc++] = arg;
+    arg += strlen(strcpy(arg, "-e;")) + 1;
+    *arg = '\0';
+    argv[argc] = NULL;
     ruby_init_without_signal_change();
-    ruby_script(argv[0]);
-    ruby_set_argv(argc, argv);
-    rb_argv0 = rb_gv_get("$PROGRAM_NAME");
     milter_manager_rubylib = g_getenv("MILTER_MANAGER_RUBYLIB");
     if (milter_manager_rubylib) {
         ruby_incpush(milter_manager_rubylib);
     }
     ruby_incpush(BINDING_LIB_DIR);
     ruby_incpush(BINDING_EXT_DIR);
-    ruby_init_loadpath();
+    ruby_process_options(argc, argv);
     load_libraries();
 #if RBGLIB_MINOR_VERSION <= 16
     g_main_context_set_poll_func(NULL, NULL);
@@ -383,6 +390,7 @@ dispose (GObject *object)
     MilterManagerRubyConfiguration *configuration;
 
     configuration = MILTER_MANAGER_RUBY_CONFIGURATION(object);
+    configuration->disposing = TRUE;
 
     G_OBJECT_CLASS(milter_manager_ruby_configuration_parent_class)->dispose(object);
 }
@@ -397,7 +405,7 @@ set_property (GObject      *object,
 
     configuration = MILTER_MANAGER_RUBY_CONFIGURATION(object);
     switch (prop_id) {
-      default:
+    default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
         break;
     }
@@ -413,7 +421,7 @@ get_property (GObject    *object,
 
     configuration = MILTER_MANAGER_RUBY_CONFIGURATION(object);
     switch (prop_id) {
-      default:
+    default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
         break;
     }
@@ -510,21 +518,23 @@ static gboolean
 real_clear (MilterManagerConfiguration *_configuration, GError **error)
 {
     MilterManagerRubyConfiguration *configuration;
-    GError *local_error = NULL;
     gboolean success = TRUE;
 
     configuration = MILTER_MANAGER_RUBY_CONFIGURATION(_configuration);
-    rb_funcall_protect(&local_error,
-                       GOBJ2RVAL(configuration),
-                       rb_intern("clear"),
-                       0);
-    if (local_error) {
-        success = FALSE;
-        if (!error) {
-            milter_error("[ruby-configuration][error][clear] %s",
-                         local_error->message);
+    if (!configuration->disposing) {
+        GError *local_error = NULL;
+        rb_funcall_protect(&local_error,
+                           GOBJ2RVAL(configuration),
+                           rb_intern("clear"),
+                           0);
+        if (local_error) {
+            success = FALSE;
+            if (!error) {
+                milter_error("[ruby-configuration][error][clear] %s",
+                             local_error->message);
+            }
+            g_propagate_error(error, local_error);
         }
-        g_propagate_error(error, local_error);
     }
 
     return success;
