@@ -1,6 +1,6 @@
 /* -*- Mode: C; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /*
- *  Copyright (C) 2008-2010  Kouhei Sutou <kou@clear-code.com>
+ *  Copyright (C) 2008-2011  Kouhei Sutou <kou@clear-code.com>
  *
  *  This library is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -31,6 +31,8 @@
 void test_replace_body (void);
 void test_replace_body_large (void);
 
+static MilterEventLoop *loop;
+
 static MilterClientContext *context;
 static MilterCommandEncoder *command_encoder;
 static MilterReplyEncoder *reply_encoder;
@@ -38,8 +40,8 @@ static MilterReplyEncoder *reply_encoder;
 static GIOChannel *channel;
 static MilterWriter *writer;
 
-static gchar *packet;
-static gsize packet_size;
+static GError *error_in_callback;
+
 static GString *expected_packet;
 
 static GString *body;
@@ -49,7 +51,8 @@ cb_end_of_message (MilterClientContext *context,
                    const gchar *chunk, gsize size,
                    gpointer user_data)
 {
-    milter_client_context_replace_body(context, body->str, body->len);
+    milter_client_context_replace_body(context, body->str, body->len,
+                                       &error_in_callback);
     milter_agent_set_writer(MILTER_AGENT(context), NULL);
 
     return MILTER_STATUS_CONTINUE;
@@ -66,40 +69,49 @@ setup_signals (MilterClientContext *context)
 #undef CONNECT
 }
 
-void
-setup (void)
+static void
+setup_option (MilterClientContext *context)
 {
+    MilterOption *option;
+
+    option = milter_option_new(2, MILTER_ACTION_CHANGE_BODY, 0);
+    milter_client_context_set_option(context, option);
+    g_object_unref(option);
+}
+
+void
+cut_setup (void)
+{
+    GError *error = NULL;
+
+    loop = milter_test_event_loop_new();
+
     context = milter_client_context_new(NULL);
+    milter_agent_set_event_loop(MILTER_AGENT(context), loop);
 
     channel = gcut_string_io_channel_new(NULL);
     g_io_channel_set_encoding(channel, NULL, NULL);
     writer = milter_writer_io_channel_new(channel);
 
     milter_agent_set_writer(MILTER_AGENT(context), writer);
-    milter_agent_start(MILTER_AGENT(context), NULL);
+    milter_agent_start(MILTER_AGENT(context), &error);
+    gcut_assert_error(error);
     setup_signals(context);
 
+    setup_option(context);
+
+    error_in_callback = NULL;
+
     command_encoder = MILTER_COMMAND_ENCODER(milter_command_encoder_new());
     reply_encoder = MILTER_REPLY_ENCODER(milter_reply_encoder_new());
-    packet = NULL;
-    packet_size = 0;
 
     expected_packet = g_string_new(NULL);
 
     body = g_string_new(NULL);
 }
 
-static void
-packet_free (void)
-{
-    if (packet)
-        g_free(packet);
-    packet = NULL;
-    packet_size = 0;
-}
-
 void
-teardown (void)
+cut_teardown (void)
 {
     if (context)
         g_object_unref(context);
@@ -114,7 +126,8 @@ teardown (void)
     if (writer)
         g_object_unref(writer);
 
-    packet_free();
+    if (loop)
+        g_object_unref(loop);
 
     if (expected_packet)
         g_string_free(expected_packet, TRUE);
@@ -126,11 +139,15 @@ teardown (void)
 typedef void (*HookFunction) (void);
 
 static GError *
-feed (void)
+feed (const gchar *packet, gsize packet_size)
 {
     GError *error = NULL;
 
+    gcut_assert_error(error_in_callback);
     milter_client_context_feed(context, packet, packet_size, &error);
+    gcut_assert_error(error_in_callback);
+
+    milter_test_pump_all_events(loop);
 
     return error;
 }
@@ -139,15 +156,16 @@ void
 test_replace_body (void)
 {
     GString *actual_data;
+    const gchar *packet;
+    gsize packet_size;
     gsize packed_size = 0;
 
     g_string_append(body, "XXX");
     milter_command_encoder_encode_end_of_message(command_encoder,
                                                  &packet, &packet_size,
                                                  NULL, 0);
-    gcut_assert_error(feed());
+    gcut_assert_error(feed(packet, packet_size));
 
-    packet_free();
     milter_reply_encoder_encode_replace_body(reply_encoder,
                                              &packet, &packet_size,
                                              body->str, body->len,
@@ -162,6 +180,8 @@ test_replace_body_large (void)
 {
     GString *actual_data;
     gsize i, rest_size;
+    const gchar *packet;
+    gsize packet_size;
     gsize packed_size = 0;
 
     for (i = 0; i < MILTER_CHUNK_SIZE + 500; i++) {
@@ -170,16 +190,14 @@ test_replace_body_large (void)
     milter_command_encoder_encode_end_of_message(command_encoder,
                                                  &packet, &packet_size,
                                                  NULL, 0);
-    gcut_assert_error(feed());
+    gcut_assert_error(feed(packet, packet_size));
 
-    packet_free();
     milter_reply_encoder_encode_replace_body(reply_encoder,
                                              &packet, &packet_size,
                                              body->str, body->len,
                                              &packed_size);
     g_string_append_len(expected_packet, packet, packet_size);
 
-    packet_free();
     rest_size = body->len - packed_size;
     milter_reply_encoder_encode_replace_body(reply_encoder,
                                              &packet, &packet_size,
