@@ -1,6 +1,6 @@
 /* -*- Mode: C; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /*
- *  Copyright (C) 2008-2010  Kouhei Sutou <kou@clear-code.com>
+ *  Copyright (C) 2008-2011  Kouhei Sutou <kou@clear-code.com>
  *
  *  This library is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -51,6 +51,7 @@ void test_suspend_time_on_unacceptable (void);
 void test_max_connections (void);
 void test_effective_user (void);
 void test_effective_group (void);
+void test_event_loop_backend (void);
 void test_maintenance_interval (void);
 void test_connection_spec (void);
 void test_need_maintain (void);
@@ -60,6 +61,11 @@ void test_need_maintain_never (void);
 void test_need_maintain_no_processing_sessions (void);
 void test_need_maintain_no_processing_sessions_below_processed_sessions (void);
 void test_need_maintain_no_processing_sessions_no_interval (void);
+void test_n_workers (void);
+void test_custom_fork (void);
+void test_default_packet_buffer_size (void);
+
+static MilterEventLoop *loop;
 
 static MilterClient *client;
 static MilterTestServer *server;
@@ -87,8 +93,6 @@ static guint shutdown_count;
 
 static const gchar *spec;
 
-static gchar *packet;
-
 static MilterOption *option;
 static MilterOption *negotiate_option;
 
@@ -123,6 +127,11 @@ static socklen_t actual_address_size;
 
 static gchar *tmp_dir;
 
+static guint loop_run_count;
+
+static guint n_worker_fork_called;
+static guint64 n_workers;
+
 static void
 cb_negotiate (MilterClientContext *context, MilterOption *option,
               MilterMacrosRequests *macros_requests, gpointer user_data)
@@ -289,6 +298,13 @@ cb_error (MilterErrorEmittable *emittable, GError *error,
     actual_error = g_error_copy(error);
 }
 
+static void
+loop_run (MilterEventLoop *loop)
+{
+    loop_run_count++;
+    milter_event_loop_run_without_custom(loop);
+}
+
 static void
 setup_client_signals (void)
 {
@@ -303,19 +319,29 @@ setup_client_signals (void)
 }
 
 void
-setup (void)
+cut_setup (void)
 {
+    const gchar *env;
+
     spec = "inet:9999@127.0.0.1";
 
     client = milter_client_new();
+    env = g_getenv("MILTER_N_WORKERS");
+    if (env && (n_workers = g_ascii_strtoull(env, NULL, 10)) > 0) {
+        if (n_workers > UINT_MAX)
+            n_workers = 0;
+        milter_client_set_n_workers(client, n_workers);
+    }
+    loop_run_count = 0;
+    loop = milter_client_get_process_loop(client);
+    g_object_ref(loop);
+    milter_event_loop_set_custom_run_func(loop, loop_run);
     setup_client_signals();
     server = NULL;
 
     decoder = milter_reply_decoder_new();
     encoder = MILTER_COMMAND_ENCODER(milter_command_encoder_new());
 
-    packet = NULL;
-
     option = NULL;
     negotiate_option = NULL;
 
@@ -366,21 +392,18 @@ setup (void)
     actual_address = NULL;
     actual_address_size = 0;
 
-    tmp_dir = g_build_filename(milter_test_get_base_dir(),
-                               "tmp",
-                               NULL);
-    cut_remove_path(tmp_dir, NULL);
-    if (g_mkdir_with_parents(tmp_dir, 0700) == -1)
-        cut_assert_errno();
+    tmp_dir = milter_test_get_tmp_dir();
+
+    n_worker_fork_called = 0;
 }
 
 void
-teardown (void)
+cut_teardown (void)
 {
     if (idle_id > 0)
-        g_source_remove(idle_id);
+        milter_event_loop_remove(loop, idle_id);
     if (idle_shutdown_id > 0)
-        g_source_remove(idle_shutdown_id);
+        milter_event_loop_remove(loop, idle_shutdown_id);
 
     if (client)
         g_object_unref(client);
@@ -392,8 +415,8 @@ teardown (void)
     if (encoder)
         g_object_unref(encoder);
 
-    if (packet)
-        g_free(packet);
+    if (loop)
+        g_object_unref(loop);
 
     if (option)
         g_object_unref(option);
@@ -465,19 +488,23 @@ setup_client (void)
     milter_client_set_connection_spec(client, spec, &error);
     gcut_assert_error(error);
 
-    idle_shutdown_id = g_idle_add_full(G_PRIORITY_LOW, cb_idle_shutdown,
-                                       NULL, NULL);
+    idle_shutdown_id = milter_event_loop_add_idle_full(loop,
+                                                       G_PRIORITY_LOW,
+                                                       cb_idle_shutdown,
+                                                       NULL,
+                                                       NULL);
 }
 
 static void
 setup_test_server (void)
 {
-    cut_trace(server = milter_test_server_new(spec, decoder));
+    cut_trace(server = milter_test_server_new(spec, decoder, loop));
 }
 
 static gboolean
 cb_idle_negotiate (gpointer user_data)
 {
+    const gchar *packet;
     gsize packet_size;
 
     cut_trace(setup_test_server());
@@ -494,6 +521,10 @@ test_negotiate (void)
     guint32 version;
     MilterActionFlags action;
     MilterStepFlags step;
+    GError *error = NULL;
+
+    if (n_workers > 0)
+        cut_omit("can't obtain the result from callbacks in child process");
 
     version = 2;
     action = MILTER_ACTION_ADD_HEADERS |
@@ -512,23 +543,24 @@ test_negotiate (void)
         MILTER_STEP_NO_END_OF_HEADER;
     option = milter_option_new(version, action, step);
 
-    idle_id = g_idle_add(cb_idle_negotiate, NULL);
+    idle_id = milter_event_loop_add_idle(loop, cb_idle_negotiate, NULL);
 
     cut_trace(setup_client());
-    if (!milter_client_main(client))
-        cut_assert_errno();
+    milter_client_run(client, &error);
+    gcut_assert_error(error);
 
     milter_assert_equal_option(option, negotiate_option);
 
     macros_requests = milter_macros_requests_new();
     milter_assert_equal_macros_requests(macros_requests,
                                         negotiate_macros_requests);
+    cut_assert_true(loop_run_count > 0);
 }
 
 static gboolean
 cb_idle_helo (gpointer user_data)
 {
-    gchar *packet;
+    const gchar *packet;
     gsize packet_size;
 
     cut_trace(setup_test_server());
@@ -541,13 +573,19 @@ cb_idle_helo (gpointer user_data)
 void
 test_helo (void)
 {
-    idle_id = g_idle_add(cb_idle_helo, NULL);
+    GError *error = NULL;
+
+    if (n_workers > 0)
+        cut_omit("can't obtain the result from callbacks in child process");
+
+    idle_id = milter_event_loop_add_idle(loop, cb_idle_helo, NULL);
 
     cut_trace(setup_client());
-    if (!milter_client_main(client))
-        cut_assert_errno();
+    milter_client_run(client, &error);
+    gcut_assert_error(error);
 
     cut_assert_equal_string(fqdn, helo_fqdn);
+    cut_assert_true(loop_run_count > 0);
 }
 
 void
@@ -555,10 +593,11 @@ test_listen_started (void)
 {
     struct sockaddr_in *address_inet;
     gchar actual_address_string[INET6_ADDRSTRLEN];
+    GError *error = NULL;
 
     cut_trace(setup_client());
-    if (!milter_client_main(client))
-        cut_assert_errno();
+    milter_client_run(client, &error);
+    gcut_assert_error(error);
 
     cut_assert_not_null(actual_address);
 
@@ -632,6 +671,7 @@ test_change_unix_socket_mode (void)
 {
     const gchar *path;
     struct stat stat_buffer;
+    GError *error = NULL;
 
     path = cut_take_printf("%s/milter.sock", tmp_dir);
     spec = cut_take_printf("unix:%s", path);
@@ -640,8 +680,8 @@ test_change_unix_socket_mode (void)
     milter_client_set_default_remove_unix_socket_on_close(client, FALSE);
 
     cut_trace(setup_client());
-    if (!milter_client_main(client))
-        cut_assert_errno();
+    milter_client_run(client, &error);
+    gcut_assert_error(error);
 
     if (stat(path, &stat_buffer) == -1)
         cut_assert_errno();
@@ -657,6 +697,7 @@ test_change_unix_socket_group (void)
     gint i, n_groups, max_n_groups;
     gid_t *groups;
     struct group *group = NULL;
+    GError *error = NULL;
 
     path = cut_take_printf("%s/milter.sock", tmp_dir);
     spec = cut_take_printf("unix:%s", path);
@@ -693,8 +734,8 @@ test_change_unix_socket_group (void)
     milter_client_set_default_remove_unix_socket_on_close(client, FALSE);
 
     cut_trace(setup_client());
-    if (!milter_client_main(client))
-        cut_assert_errno();
+    milter_client_run(client, &error);
+    gcut_assert_error(error);
 
     if (stat(path, &stat_buffer) == -1)
         cut_assert_errno();
@@ -721,7 +762,8 @@ test_remove_unix_socket_on_close (void)
     spec = cut_take_printf("unix:%s", path);
 
     cut_trace(setup_client());
-    cut_assert_true(milter_client_main(client));
+    cut_assert_true(milter_client_run(client, &actual_error));
+    gcut_assert_error(actual_error);
 
     cut_assert_false(g_file_test(path, G_FILE_TEST_EXISTS));
 }
@@ -747,8 +789,9 @@ test_remove_unix_socket_on_create (void)
     gcut_assert_error(error);
 
     cut_trace(setup_client());
-    cut_assert_true(milter_client_main(client));
-    cut_assert_null(actual_error);
+    cut_assert_true(milter_client_run(client, &error));
+    gcut_assert_error(error);
+    gcut_assert_error(actual_error);
 }
 
 void
@@ -766,8 +809,10 @@ test_not_remove_unix_socket_on_create (void)
     milter_client_set_remove_unix_socket_on_create(client, FALSE);
 
     cut_trace(setup_client());
-    cut_assert_false(milter_client_main(client));
+    cut_assert_false(milter_client_run(client, &error));
+    gcut_assert_error(actual_error);
 
+    actual_error = error;
     expected_error = g_error_new(MILTER_CONNECTION_ERROR,
                                  MILTER_CONNECTION_ERROR_BIND_FAILURE,
                                  "failed to bind(): %s: %s",
@@ -827,6 +872,21 @@ test_effective_group (void)
         "nogroup", milter_client_get_effective_group(client));
 }
 
+void
+test_event_loop_backend (void)
+{
+    gcut_assert_equal_enum(
+        MILTER_TYPE_CLIENT_EVENT_LOOP_BACKEND,
+        MILTER_CLIENT_EVENT_LOOP_BACKEND_GLIB,
+        milter_client_get_event_loop_backend(client));
+    milter_client_set_event_loop_backend(client,
+                                         MILTER_CLIENT_EVENT_LOOP_BACKEND_LIBEV);
+    gcut_assert_equal_enum(
+        MILTER_TYPE_CLIENT_EVENT_LOOP_BACKEND,
+        MILTER_CLIENT_EVENT_LOOP_BACKEND_LIBEV,
+        milter_client_get_event_loop_backend(client));
+}
+
 void
 test_maintenance_interval (void)
 {
@@ -913,6 +973,52 @@ test_need_maintain_no_processing_sessions_no_interval (void)
     cut_assert_true(milter_client_need_maintain(client, 1));
 }
 
+void
+test_n_workers (void)
+{
+    cut_assert_equal_uint(
+        n_workers, milter_client_get_n_workers(client));
+    milter_client_set_n_workers(client, 10);
+    cut_assert_equal_uint(
+        10, milter_client_get_n_workers(client));
+}
+
+static GPid
+worker_fork (MilterClient *loop)
+{
+    n_worker_fork_called++;
+    return 29;
+}
+
+void
+test_custom_fork (void)
+{
+    milter_client_set_custom_fork_func(client, worker_fork);
+    cut_assert_equal_uint(29, milter_client_fork(client));
+    cut_assert_equal_uint(1, n_worker_fork_called);
+}
+
+void
+test_default_packet_buffer_size (void)
+{
+    MilterClientContext *context;
+
+    cut_assert_equal_uint(0,
+                          milter_client_get_default_packet_buffer_size(client));
+    context = milter_client_create_context(client);
+    gcut_take_object(G_OBJECT(context));
+    cut_assert_equal_uint(0,
+                          milter_client_context_get_packet_buffer_size(context));
+
+    milter_client_set_default_packet_buffer_size(client, 4096);
+    cut_assert_equal_uint(4096,
+                          milter_client_get_default_packet_buffer_size(client));
+    context = milter_client_create_context(client);
+    gcut_take_object(G_OBJECT(context));
+    cut_assert_equal_uint(4096,
+                          milter_client_context_get_packet_buffer_size(context));
+}
+
 /*
 vi:ts=4:nowrap:ai:expandtab:sw=4
 */
