@@ -465,9 +465,9 @@ data_flags_names (void)
     ADD("multi", MILTER_LOG_LEVEL_CRITICAL | MILTER_LOG_LEVEL_INFO,
         "critical|info");
     ADD("all", MILTER_LOG_LEVEL_ALL,
-        "critical|error|warning|message|info|debug|statistics");
-    ADD("unknown", MILTER_LOG_LEVEL_STATISTICS << 1,
-        "(unknown flags: MilterLogLevelFlags:0x100)");
+        "critical|error|warning|message|info|debug|statistics|profile");
+    ADD("unknown", MILTER_LOG_LEVEL_PROFILE << 1,
+        "(unknown flags: MilterLogLevelFlags:0x200)");
 
 #undef ADD
 }
