@@ -1,6 +1,6 @@
 /* -*- Mode: C; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /*
- *  Copyright (C) 2008-2009  Kouhei Sutou <kou@cozmixng.org>
+ *  Copyright (C) 2008-2011  Kouhei Sutou <kou@clear-code.com>
  *
  *  This library is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -37,6 +37,8 @@ void test_set_configuration (void);
 void test_set_configuration_failed (void);
 void test_reload (void);
 
+static MilterEventLoop *loop;
+
 static MilterManager *manager;
 static MilterManagerControllerContext *context;
 
@@ -49,9 +51,6 @@ static GIOChannel *output_channel;
 static MilterManagerControlCommandEncoder *command_encoder;
 static MilterManagerControlReplyEncoder *reply_encoder;
 
-static gchar *packet;
-static gsize packet_size;
-
 static gchar *tmp_dir;
 static gchar *custom_config_path;
 
@@ -70,7 +69,7 @@ setup_input_io (void)
     g_object_unref(input_reader);
 
     writer = milter_writer_io_channel_new(channel);
-    milter_writer_start(writer, NULL);
+    milter_writer_start(writer, loop);
 
     g_io_channel_unref(channel);
 }
@@ -92,21 +91,27 @@ setup_output_io (void)
 static void
 setup_io (void)
 {
+    GError *error = NULL;
+
     setup_input_io();
     setup_output_io();
-    milter_agent_start(MILTER_AGENT(context), NULL);
+    milter_agent_start(MILTER_AGENT(context), &error);
+    gcut_assert_error(error);
 }
 
 void
-setup (void)
+cut_setup (void)
 {
     MilterManagerConfiguration *config;
     MilterEncoder *encoder;
 
+    loop = milter_test_event_loop_new();
+
     config = milter_manager_configuration_new(NULL);
     manager = milter_manager_new(config);
     g_object_unref(config);
     context = milter_manager_controller_context_new(manager);
+    milter_agent_set_event_loop(MILTER_AGENT(context), loop);
 
     setup_io();
 
@@ -118,9 +123,6 @@ setup (void)
     expected_error = NULL;
     actual_error = NULL;
 
-    packet = NULL;
-    packet_size = 0;
-
     tmp_dir = g_build_filename(milter_test_get_base_dir(),
                                "tmp",
                                NULL);
@@ -134,7 +136,7 @@ setup (void)
 }
 
 void
-teardown (void)
+cut_teardown (void)
 {
     if (manager)
         g_object_unref(manager);
@@ -151,14 +153,14 @@ teardown (void)
     if (reply_encoder)
         g_object_unref(reply_encoder);
 
+    if (loop)
+        g_object_unref(loop);
+
     if (actual_error)
         g_error_free(actual_error);
     if (expected_error)
         g_error_free(expected_error);
 
-    if (packet)
-        g_free(packet);
-
     if (tmp_dir) {
         g_chmod(tmp_dir, 0700);
         cut_remove_path(tmp_dir, NULL);
@@ -169,12 +171,24 @@ teardown (void)
         g_free(custom_config_path);
 }
 
+static void
+write_packet (const gchar *packet, gsize packet_size)
+{
+    GError *error = NULL;
+
+    milter_writer_write(writer, packet, packet_size, &error);
+    gcut_assert_error(error);
+    milter_writer_flush(writer, &error);
+    gcut_assert_error(error);
+}
+
 void
 test_set_configuration (void)
 {
+    const gchar *packet;
+    gsize packet_size;
     const gchar configuration[] = "XXX";
     GString *output;
-    GError *error = NULL;
 
     cut_assert_path_not_exist(custom_config_path);
 
@@ -182,13 +196,11 @@ test_set_configuration (void)
         command_encoder,
         &packet, &packet_size,
         configuration, strlen(configuration));
-    milter_writer_write(writer, packet, packet_size, NULL, &error);
-    gcut_assert_error(error);
-    milter_test_pump_all_events();
+    cut_trace(write_packet(packet, packet_size));
+    milter_test_pump_all_events(loop);
 
     cut_assert_path_exist(custom_config_path);
 
-    g_free(packet);
     milter_manager_control_reply_encoder_encode_success(reply_encoder,
                                                         &packet, &packet_size);
     output = gcut_string_io_channel_get_string(output_channel);
@@ -200,10 +212,11 @@ void
 test_set_configuration_failed (void)
 {
     MilterManagerConfiguration *config;
+    const gchar *packet;
+    gsize packet_size;
     const gchar configuration[] = "XXX";
     GString *output;
     guint packet_size_space;
-    GError *error = NULL;
 
     config = milter_manager_get_configuration(manager);
     milter_manager_configuration_clear_load_paths(config);
@@ -216,13 +229,11 @@ test_set_configuration_failed (void)
         command_encoder,
         &packet, &packet_size,
         configuration, strlen(configuration));
-    milter_writer_write(writer, packet, packet_size, NULL, &error);
-    gcut_assert_error(error);
-    milter_test_pump_all_events();
+    cut_trace(write_packet(packet, packet_size));
+    milter_test_pump_all_events(loop);
 
     cut_assert_path_not_exist(custom_config_path);
 
-    g_free(packet);
     milter_manager_control_reply_encoder_encode_error(
         reply_encoder,
         &packet, &packet_size,
@@ -247,6 +258,8 @@ void
 test_reload (void)
 {
     MilterManagerConfiguration *config;
+    const gchar *packet;
+    gsize packet_size;
     GString *output;
     GError *error = NULL;
 
@@ -265,12 +278,10 @@ test_reload (void)
 
     milter_manager_control_command_encoder_encode_reload(command_encoder,
                                                          &packet, &packet_size);
-    milter_writer_write(writer, packet, packet_size, NULL, &error);
-    gcut_assert_error(error);
-    milter_test_pump_all_events();
+    cut_trace(write_packet(packet, packet_size));
+    milter_test_pump_all_events(loop);
     cut_assert_true(milter_manager_configuration_is_privilege_mode(config));
 
-    g_free(packet);
     milter_manager_control_reply_encoder_encode_success(reply_encoder,
                                                         &packet, &packet_size);
     output = gcut_string_io_channel_get_string(output_channel);
