@@ -1,6 +1,6 @@
 /* -*- Mode: C; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /*
- *  Copyright (C) 2008-2010  Kouhei Sutou <kou@clear-code.com>
+ *  Copyright (C) 2008-2011  Kouhei Sutou <kou@clear-code.com>
  *
  *  This library is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -42,6 +42,8 @@ void test_configuration (void);
 static gchar *scenario_dir;
 static MilterManagerTestScenario *main_scenario;
 
+static MilterEventLoop *loop;
+
 static MilterManagerConfiguration *config;
 static MilterClientContext *client_context;
 static MilterManagerLeader *leader;
@@ -246,20 +248,23 @@ setup_client_context_signals (MilterClientContext *context)
 }
 
 void
-setup (void)
+cut_setup (void)
 {
-    GIOChannel *channel;
-    MilterWriter *writer;
+    GIOChannel *channel, *null_channel;
+    MilterWriter *writer, *null_writer;
     MilterReader *reader;
     GError *error = NULL;
 
-    scenario_dir = g_build_filename(milter_test_get_base_dir(),
+    scenario_dir = g_build_filename(milter_test_get_source_dir(),
+                                    "test",
                                     "fixtures",
                                     "leader",
                                     NULL);
 
     main_scenario = NULL;
 
+    loop = milter_test_event_loop_new();
+
     config = milter_manager_configuration_new(NULL);
     milter_manager_configuration_load(config, "milter-manager.conf", &error);
     gcut_assert_error(error);
@@ -275,12 +280,24 @@ setup (void)
     client_context = milter_client_context_new(NULL);
     milter_agent_set_writer(MILTER_AGENT(client_context), writer);
     g_object_unref(writer);
-    milter_agent_start(MILTER_AGENT(client_context), NULL);
+    milter_agent_set_event_loop(MILTER_AGENT(client_context), loop);
+    milter_agent_start(MILTER_AGENT(client_context), &error);
+    gcut_assert_error(error);
+
+    null_channel = gcut_string_io_channel_new(NULL);
+    g_io_channel_set_encoding(null_channel, NULL, NULL);
+    g_io_channel_set_flags(null_channel, G_IO_FLAG_NONBLOCK, NULL);
+    null_writer = milter_writer_io_channel_new(null_channel);
+    g_io_channel_unref(null_channel);
 
     server = milter_manager_test_server_new();
     milter_agent_set_reader(MILTER_AGENT(server), reader);
     g_object_unref(reader);
-    milter_agent_start(MILTER_AGENT(server), NULL);
+    milter_agent_set_writer(MILTER_AGENT(server), null_writer);
+    g_object_unref(null_writer);
+    milter_agent_set_event_loop(MILTER_AGENT(server), loop);
+    milter_agent_start(MILTER_AGENT(server), &error);
+    gcut_assert_error(error);
 
     leader = milter_manager_leader_new(config, client_context);
     actual_error = NULL;
@@ -320,6 +337,9 @@ cut_teardown (void)
     if (server)
         g_object_unref(server);
 
+    if (loop)
+        g_object_unref(loop);
+
     if (scenario_dir)
         g_free(scenario_dir);
     if (main_scenario)
@@ -331,6 +351,33 @@ cut_teardown (void)
         g_error_free(expected_error);
 }
 
+#define pump_all_events_without_error_check()           \
+    cut_trace_with_info_expression(                     \
+        pump_all_events_without_error_check_helper(),   \
+        pump_all_events_without_error_check())
+
+#define pump_all_events()                       \
+    cut_trace_with_info_expression(             \
+        pump_all_events_helper(),               \
+        pump_all_events())
+
+static void
+pump_all_events_without_error_check_helper (void)
+{
+    milter_test_pump_all_events(loop);
+}
+
+static void
+pump_all_events_helper (void)
+{
+    GError *error;
+
+    pump_all_events_without_error_check_helper();
+    error = actual_error;
+    actual_error = NULL;
+    gcut_assert_error(error);
+}
+
 #define started_clients                                                 \
     milter_manager_test_scenario_get_started_clients(main_scenario)
 
@@ -415,13 +462,14 @@ assert_have_response_helper (const gchar *name)
     gboolean timeout_emitted = FALSE;
     guint timeout_emitted_id;
 
-    timeout_emitted_id = g_timeout_add(3000, cb_timeout_emitted,
-                                       &timeout_emitted);
+    timeout_emitted_id =
+        milter_event_loop_add_timeout(loop, 3,
+                                      cb_timeout_emitted, &timeout_emitted);
     while (!timeout_emitted &&
            get_response_count(name) == 0) {
-        g_main_context_iteration(NULL, TRUE);
+        milter_event_loop_iterate(loop, TRUE);
     }
-    g_source_remove(timeout_emitted_id);
+    milter_event_loop_remove(loop, timeout_emitted_id);
 
     cut_set_message("<%s>", name);
     cut_assert_false(timeout_emitted);
@@ -453,12 +501,13 @@ wait_finished_helper (void)
     gboolean timeout_emitted = FALSE;
     guint timeout_emitted_id;
 
-    timeout_emitted_id = g_timeout_add(100, cb_timeout_emitted,
-                                       &timeout_emitted);
+    timeout_emitted_id =
+        milter_event_loop_add_timeout(loop, 0.1,
+                                      cb_timeout_emitted, &timeout_emitted);
     while (!timeout_emitted && !finished) {
-        g_main_context_iteration(NULL, TRUE);
+        milter_event_loop_iterate(loop, TRUE);
     }
-    g_source_remove(timeout_emitted_id);
+    milter_event_loop_remove(loop, timeout_emitted_id);
 
     cut_assert_false(timeout_emitted);
     cut_assert_true(finished);
@@ -475,12 +524,15 @@ wait_leader_error_helper (void)
     gboolean timeout_emitted = FALSE;
     guint timeout_emitted_id;
 
-    timeout_emitted_id = g_timeout_add(100, cb_timeout_emitted,
-                                       &timeout_emitted);
+    pump_all_events_without_error_check();
+
+    timeout_emitted_id =
+        milter_event_loop_add_timeout(loop, 0.1,
+                                      cb_timeout_emitted, &timeout_emitted);
     while (!timeout_emitted && !actual_error) {
-        g_main_context_iteration(NULL, TRUE);
+        milter_event_loop_iterate(loop, TRUE);
     }
-    g_source_remove(timeout_emitted_id);
+    milter_event_loop_remove(loop, timeout_emitted_id);
 
     cut_assert_false(timeout_emitted);
     cut_assert_not_null(actual_error);
@@ -599,6 +651,8 @@ assert_response_error (MilterManagerTestScenario *scenario, const gchar *group)
         message = get_string(scenario, group, "error_message");
         expected_error = g_error_new(domain, code, "%s", message);
         wait_leader_error();
+    } else {
+        pump_all_events();
     }
 
     gcut_assert_equal_error(expected_error, actual_error);
@@ -691,7 +745,7 @@ assert_response_common (MilterManagerTestScenario *scenario, const gchar *group)
         assert_n_received(scenario, group, "n_abort", "abort");
     }
 
-    g_main_context_iteration(NULL, FALSE);
+    milter_event_loop_iterate(loop, FALSE);
     cut_set_message("[%s]", group);
     gcut_assert_equal_enum(MILTER_TYPE_STATUS, status, response_status);
 
@@ -703,7 +757,7 @@ assert_response_reply_code (MilterManagerTestScenario *scenario, const gchar *gr
     const GList *replied_codes;
 
     replied_codes = get_string_g_list(scenario, group, "reply_codes");
-    cut_trace(milter_manager_test_wait_signal(FALSE));
+    pump_all_events();
     gcut_assert_equal_list_string(
         replied_codes,
         milter_manager_test_server_get_received_reply_codes(server));
@@ -788,8 +842,6 @@ do_negotiate (MilterManagerTestScenario *scenario, const gchar *group)
     MilterOption *option;
     const GList *options;
 
-    milter_server_context_set_state(MILTER_SERVER_CONTEXT(server),
-                                    MILTER_SERVER_CONTEXT_STATE_NEGOTIATE);
     option = milter_manager_test_scenario_get_option(scenario, group,
                                                      main_scenario);
     gcut_take_object(G_OBJECT(option));
@@ -845,8 +897,6 @@ do_connect (MilterManagerTestScenario *scenario, const gchar *group)
     const GList *expected_connect_infos;
     const GList *actual_connect_infos;
 
-    milter_server_context_set_state(MILTER_SERVER_CONTEXT(server),
-                                    MILTER_SERVER_CONTEXT_STATE_CONNECT);
     host = get_string(scenario, group, "host");
     address_spec = get_string(scenario, group, "address");
 
@@ -855,6 +905,8 @@ do_connect (MilterManagerTestScenario *scenario, const gchar *group)
                                  &error);
     gcut_assert_error(error);
 
+    milter_server_context_connect(MILTER_SERVER_CONTEXT(server),
+                                  host, address, address_size);
     define_macro(scenario, group, MILTER_COMMAND_CONNECT);
     milter_manager_leader_connect(leader, host, address, address_size);
     g_free(address);
@@ -880,9 +932,8 @@ do_helo (MilterManagerTestScenario *scenario, const gchar *group)
     const gchar *fqdn;
     const GList *expected_fqdns;
 
-    milter_server_context_set_state(MILTER_SERVER_CONTEXT(server),
-                                    MILTER_SERVER_CONTEXT_STATE_HELO);
     fqdn = get_string(scenario, group, "fqdn");
+    milter_server_context_helo(MILTER_SERVER_CONTEXT(server), fqdn);
     milter_manager_leader_helo(leader, fqdn);
 
     assert_response(scenario, group);
@@ -900,9 +951,8 @@ do_envelope_from (MilterManagerTestScenario *scenario, const gchar *group)
     const gchar *from;
     const GList *expected_froms;
 
-    milter_server_context_set_state(MILTER_SERVER_CONTEXT(server),
-                                    MILTER_SERVER_CONTEXT_STATE_ENVELOPE_FROM);
     from = get_string(scenario, group, "from");
+    milter_server_context_envelope_from(MILTER_SERVER_CONTEXT(server), from);
     define_macro(scenario, group, MILTER_COMMAND_ENVELOPE_FROM);
     milter_manager_leader_envelope_from(leader, from);
 
@@ -923,10 +973,10 @@ do_envelope_recipient (MilterManagerTestScenario *scenario, const gchar *group)
     const gchar *recipient;
     const GList *expected_recipients;
 
-    milter_server_context_set_state(MILTER_SERVER_CONTEXT(server),
-                                    MILTER_SERVER_CONTEXT_STATE_ENVELOPE_RECIPIENT);
     recipient = get_string(scenario, group, "recipient");
     n_envelope_recipient_responses = 0;
+    milter_server_context_envelope_recipient(MILTER_SERVER_CONTEXT(server),
+                                             recipient);
     milter_manager_leader_envelope_recipient(leader, recipient);
 
     assert_response(scenario, group);
@@ -941,8 +991,7 @@ do_envelope_recipient (MilterManagerTestScenario *scenario, const gchar *group)
 static void
 do_data (MilterManagerTestScenario *scenario, const gchar *group)
 {
-    milter_server_context_set_state(MILTER_SERVER_CONTEXT(server),
-                                    MILTER_SERVER_CONTEXT_STATE_DATA);
+    milter_server_context_data(MILTER_SERVER_CONTEXT(server));
     milter_manager_leader_data(leader);
 
     assert_response(scenario, group);
@@ -955,11 +1004,10 @@ do_header (MilterManagerTestScenario *scenario, const gchar *group)
     const GList *actual_headers;
     const gchar *name, *value;
 
-    milter_server_context_set_state(MILTER_SERVER_CONTEXT(server),
-                                    MILTER_SERVER_CONTEXT_STATE_HEADER);
     name = get_string(scenario, group, "name");
     value = get_string(scenario, group, "value");
     n_header_responses = 0;
+    milter_server_context_header(MILTER_SERVER_CONTEXT(server), name, value);
     milter_manager_leader_header(leader, name, value);
 
     milter_manager_test_server_add_header(server, name, value);
@@ -981,8 +1029,7 @@ do_header (MilterManagerTestScenario *scenario, const gchar *group)
 static void
 do_end_of_header (MilterManagerTestScenario *scenario, const gchar *group)
 {
-    milter_server_context_set_state(MILTER_SERVER_CONTEXT(server),
-                                    MILTER_SERVER_CONTEXT_STATE_END_OF_HEADER);
+    milter_server_context_end_of_header(MILTER_SERVER_CONTEXT(server));
     milter_manager_leader_end_of_header(leader);
 
     assert_response(scenario, group);
@@ -1058,9 +1105,9 @@ do_body (MilterManagerTestScenario *scenario, const gchar *group)
     gsize chunk_size = 0;
     const GList *expected_chunks, *actual_chunks;
 
-    milter_server_context_set_state(MILTER_SERVER_CONTEXT(server),
-                                    MILTER_SERVER_CONTEXT_STATE_BODY);
     get_chunk(scenario, group, &chunk, &chunk_size);
+    milter_server_context_body(MILTER_SERVER_CONTEXT(server),
+                               chunk, chunk_size);
     n_body_responses = 0;
     milter_manager_leader_body(leader, chunk, chunk_size);
 
@@ -1080,10 +1127,10 @@ do_end_of_message (MilterManagerTestScenario *scenario, const gchar *group)
     gsize chunk_size = 0;
     const GList *expected_chunks, *actual_chunks;
 
-    milter_server_context_set_state(MILTER_SERVER_CONTEXT(server),
-                                    MILTER_SERVER_CONTEXT_STATE_END_OF_MESSAGE);
     get_chunk(scenario, group, &chunk, &chunk_size);
 
+    milter_server_context_end_of_message(MILTER_SERVER_CONTEXT(server),
+                                         chunk, chunk_size);
     milter_client_context_set_state(client_context,
                                     MILTER_CLIENT_CONTEXT_STATE_END_OF_MESSAGE);
     milter_manager_leader_end_of_message(leader, chunk, chunk_size);
@@ -1232,7 +1279,7 @@ static void
 do_end_of_message_full (MilterManagerTestScenario *scenario, const gchar *group)
 {
     cut_trace(do_end_of_message(scenario, group));
-    cut_trace(milter_manager_test_wait_signal(FALSE));
+    cut_trace(milter_manager_test_wait_signal(loop, FALSE));
     cut_trace(do_end_of_message_quarantine(scenario, group));
     cut_trace(do_end_of_message_header(scenario, group));
     cut_trace(do_end_of_message_change_from(scenario, group));
@@ -1729,7 +1776,7 @@ test_scenario (gconstpointer data)
         gcut_assert_error(error);
     }
 
-    cut_trace(milter_manager_test_scenario_start_clients(main_scenario));
+    cut_trace(milter_manager_test_scenario_start_clients(main_scenario, loop));
     cut_trace(do_actions(main_scenario));
 }
 
