@@ -1,6 +1,6 @@
 /* -*- Mode: C; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /*
- *  Copyright (C) 2008-2010  Kouhei Sutou <kou@cozmixng.org>
+ *  Copyright (C) 2008-2011  Kouhei Sutou <kou@clear-code.com>
  *
  *  This library is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -48,6 +48,8 @@ void test_remove_manager_unix_socket_on_close (void);
 void data_scenario (void);
 void test_scenario (gconstpointer data);
 
+static MilterEventLoop *loop;
+
 static gchar *scenario_dir;
 static MilterManagerTestScenario *main_scenario;
 
@@ -72,8 +74,7 @@ static EggData *server_data;
 static gchar *
 build_manager_path (void)
 {
-    return g_build_filename(milter_test_get_base_dir(),
-                            "..",
+    return g_build_filename(milter_test_get_build_dir(),
                             "src",
                             "milter-manager",
                             NULL);
@@ -82,8 +83,7 @@ build_manager_path (void)
 static gchar *
 build_server_path (void)
 {
-    return g_build_filename(milter_test_get_base_dir(),
-                            "..",
+    return g_build_filename(milter_test_get_build_dir(),
                             "tool",
                             "milter-test-server",
                             NULL);
@@ -130,10 +130,12 @@ wait_for_reaping (EggData *data, gboolean must)
     gboolean timeout_emitted = FALSE;
     guint timeout_id;
 
-    timeout_id = g_timeout_add(2000, cb_timeout_emitted, &timeout_emitted);
+    timeout_id = milter_event_loop_add_timeout(loop, 2,
+                                               cb_timeout_emitted,
+                                               &timeout_emitted);
     while (!timeout_emitted && !data->reaped)
-        g_main_context_iteration(NULL, TRUE);
-    g_source_remove(timeout_id);
+        milter_event_loop_iterate(loop, TRUE);
+    milter_event_loop_remove(loop, timeout_id);
 
     if (must) {
         cut_set_message("OUTPUT: <%s>\n"
@@ -168,10 +170,13 @@ cut_setup (void)
 {
     gchar *lt_milter_manager;
 
+    loop = milter_test_event_loop_new();
+
     original_lang = g_strdup(g_getenv("LANG"));
     g_setenv("LANG", "C", TRUE);
 
-    scenario_dir = g_build_filename(milter_test_get_base_dir(),
+    scenario_dir = g_build_filename(milter_test_get_source_dir(),
+                                    "test",
                                     "fixtures",
                                     "manager",
                                     NULL);
@@ -181,8 +186,7 @@ cut_setup (void)
     manager_data = egg_data_new(build_manager_path);
     server_data = egg_data_new(build_server_path);
 
-    lt_milter_manager = g_build_filename(milter_test_get_base_dir(),
-                                         "..",
+    lt_milter_manager = g_build_filename(milter_test_get_build_dir(),
                                          "src",
                                          ".libs",
                                          "lt-milter-manager",
@@ -193,12 +197,7 @@ cut_setup (void)
         milter_manager_program_name = g_strdup("milter-manager");
     g_free(lt_milter_manager);
 
-    tmp_dir = g_build_filename(milter_test_get_base_dir(),
-                               "tmp",
-                               NULL);
-    cut_remove_path(tmp_dir, NULL);
-    if (g_mkdir_with_parents(tmp_dir, 0700) == -1)
-        cut_assert_errno();
+    tmp_dir = milter_test_get_tmp_dir();
 }
 
 void
@@ -228,6 +227,9 @@ cut_teardown (void)
     } else {
         g_unsetenv("LANG");
     }
+
+    if (loop)
+        g_object_unref(loop);
 }
 
 static void
@@ -311,8 +313,9 @@ wait_for_manager_ready (const gchar *spec)
                                  &error);
     gcut_assert_error(error);
 
-    timeout_waiting_id = g_timeout_add(1000, cb_timeout_emitted,
-                                       &timeout_emitted);
+    timeout_waiting_id = milter_event_loop_add_timeout(loop, 1,
+                                                       cb_timeout_emitted,
+                                                       &timeout_emitted);
     do {
         if (socket_fd != -1)
             close(socket_fd);
@@ -326,9 +329,9 @@ wait_for_manager_ready (const gchar *spec)
         } else {
             errno_keep = errno;
         }
-        g_main_context_iteration(NULL, FALSE);
+        milter_event_loop_iterate(loop, FALSE);
     } while (!timeout_emitted);
-    g_source_remove(timeout_waiting_id);
+    milter_event_loop_remove(loop, timeout_waiting_id);
     if (socket_fd != -1)
         close(socket_fd);
 
@@ -522,7 +525,7 @@ test_scenario (gconstpointer data)
                                   MILTER_MANAGER_TEST_SCENARIO_GROUP_NAME,
                                   omit_key));
 
-    cut_trace(milter_manager_test_scenario_start_clients(main_scenario));
+    cut_trace(milter_manager_test_scenario_start_clients(main_scenario, loop));
 
     start_server();
 
