@@ -1,6 +1,6 @@
 /* -*- Mode: C; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /*
- *  Copyright (C) 2008-2009  Kouhei Sutou <kou@clear-code.com>
+ *  Copyright (C) 2008-2011  Kouhei Sutou <kou@clear-code.com>
  *
  *  This library is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -48,7 +48,6 @@ void test_quit (void);
 void test_quit_after_connect (void);
 void test_abort (void);
 void test_abort_and_quit (void);
-void test_reading_timeout (void);
 
 static guint32 protocol_version;
 
@@ -59,16 +58,11 @@ static MilterWriter *writer;
 static MilterDecoder *decoder;
 static MilterCommandEncoder *encoder;
 static MilterReplyEncoder *reply_encoder;
-static gchar *packet;
-static gsize packet_size;
 static GString *packet_string;
 static GHashTable *macros;
 
 static GIOChannel *channel;
-
-static gboolean timed_out_before;
-static gboolean timed_out_after;
-static guint timeout_id;
+static MilterEventLoop *loop;
 
 static GError *expected_error, *actual_error;
 
@@ -109,9 +103,10 @@ setup_signals (MilterServerContext *context)
 }
 
 void
-setup (void)
+cut_setup (void)
 {
     MilterOption *option;
+    GError *error = NULL;
 
     protocol_version = 6;
 
@@ -136,23 +131,20 @@ setup (void)
     channel = gcut_string_io_channel_new(NULL);
     g_io_channel_set_encoding(channel, NULL, NULL);
 
+    loop = milter_test_event_loop_new();
+    milter_agent_set_event_loop(MILTER_AGENT(context), loop);
     writer = milter_writer_io_channel_new(channel);
     milter_agent_set_writer(MILTER_AGENT(context), writer);
-    milter_agent_start(MILTER_AGENT(context), NULL);
+    milter_agent_start(MILTER_AGENT(context), &error);
+    gcut_assert_error(error);
 
     decoder = milter_agent_get_decoder(MILTER_AGENT(context));
 
     encoder = MILTER_COMMAND_ENCODER(milter_command_encoder_new());
     reply_encoder = MILTER_REPLY_ENCODER(milter_reply_encoder_new());
-    packet = NULL;
-    packet_size = 0;
     packet_string = NULL;
     macros = NULL;
 
-    timed_out_before = FALSE;
-    timed_out_after = FALSE;
-    timeout_id = 0;
-
     expected_error = NULL;
     actual_error = NULL;
 
@@ -161,15 +153,6 @@ setup (void)
     expected_result = NULL;
 }
 
-static void
-packet_free (void)
-{
-    if (packet)
-        g_free(packet);
-    packet = NULL;
-    packet_size = 0;
-}
-
 static void
 channel_free (void)
 {
@@ -177,17 +160,17 @@ channel_free (void)
 }
 
 void
-teardown (void)
+cut_teardown (void)
 {
-    if (timeout_id > 0)
-        g_source_remove(timeout_id);
-
     if (context)
         g_object_unref(context);
 
     if (writer)
         g_object_unref(writer);
 
+    if (loop)
+        g_object_unref(loop);
+
     if (encoder)
         g_object_unref(encoder);
     if (reply_encoder)
@@ -196,7 +179,6 @@ teardown (void)
     if (channel)
         g_io_channel_unref(channel);
 
-    packet_free();
     if (packet_string)
         g_string_free(packet_string, TRUE);
     if (macros)
@@ -213,12 +195,23 @@ teardown (void)
         g_object_unref(expected_result);
 }
 
+static void
+pump_all_events (void)
+{
+    GError *error;
+
+    milter_test_pump_all_events(loop);
+    error = actual_error;
+    actual_error = NULL;
+    gcut_assert_error(error);
+}
+
 #define milter_test_assert_packet(channel, packet, packet_size)         \
     cut_trace(milter_test_assert_packet_helper(channel, packet, packet_size))
 
 static void
 milter_test_assert_packet_helper (GIOChannel *channel,
-                                  gchar *packet, gsize packet_size)
+                                  const gchar *packet, gsize packet_size)
 {
     GString *actual;
 
@@ -334,12 +327,15 @@ void
 test_negotiate (void)
 {
     MilterOption *option;
+    const gchar *packet;
+    gsize packet_size;
 
     option = milter_option_new(protocol_version,
                                MILTER_ACTION_ADD_HEADERS,
                                MILTER_STEP_NONE);
     gcut_take_object(G_OBJECT(option));
     cut_assert_true(milter_server_context_negotiate(context, option));
+    pump_all_events();
     milter_test_assert_state(NEGOTIATE);
     milter_test_assert_status(NOT_CHANGE);
 
@@ -356,11 +352,14 @@ test_negotiated_newer_version (void)
 {
     MilterOption *option, *reply_option;
     MilterMacrosRequests *macros_requests;
+    const gchar *packet;
+    gsize packet_size;
     GError *error = NULL;
 
     option = milter_option_new(2, MILTER_ACTION_ADD_HEADERS, MILTER_STEP_NONE);
     gcut_take_object(G_OBJECT(option));
     cut_assert_true(milter_server_context_negotiate(context, option));
+    pump_all_events();
     milter_test_assert_state(NEGOTIATE);
     milter_test_assert_status(NOT_CHANGE);
 
@@ -391,6 +390,8 @@ reply_negotiate (void)
 {
     MilterOption *reply_option;
     MilterMacrosRequests *macros_requests;
+    const gchar *packet;
+    gsize packet_size;
     GError *error = NULL;
 
     reply_option = milter_option_new(protocol_version,
@@ -405,18 +406,18 @@ reply_negotiate (void)
 
     milter_decoder_decode(decoder, packet, packet_size, &error);
     gcut_assert_error(error);
-    packet_free();
 }
 
 static void
 reply_continue (void)
 {
+    const gchar *packet;
+    gsize packet_size;
     GError *error = NULL;
 
     milter_reply_encoder_encode_continue(reply_encoder, &packet, &packet_size);
     milter_decoder_decode(decoder, packet, packet_size, &error);
     gcut_assert_error(error);
-    packet_free();
 }
 
 void
@@ -426,9 +427,10 @@ test_connect (void)
     const gchar host_name[] = "mx.local.net";
     const gchar ip_address[] = "192.168.123.123";
     guint16 port;
+    const gchar *packet;
+    gsize packet_size;
 
     test_negotiate();
-    packet_free();
     channel_free();
 
     reply_negotiate();
@@ -441,6 +443,7 @@ test_connect (void)
     cut_assert_true(milter_server_context_connect(context, host_name,
                                                   (struct sockaddr *)&address,
                                                   sizeof(address)));
+    pump_all_events();
     milter_test_assert_state(CONNECT);
     milter_test_assert_status(NOT_CHANGE);
 
@@ -461,9 +464,10 @@ test_connect_with_macro (void)
     const gchar host_name[] = "mx.local.net";
     const gchar ip_address[] = "192.168.123.123";
     guint16 port;
+    const gchar *packet;
+    gsize packet_size;
 
     test_negotiate();
-    packet_free();
     channel_free();
 
     reply_negotiate();
@@ -487,11 +491,11 @@ test_connect_with_macro (void)
     cut_assert_true(milter_server_context_connect(context, host_name,
                                                   (struct sockaddr *)&address,
                                                   sizeof(address)));
+    pump_all_events();
     milter_test_assert_state(CONNECT);
     milter_test_assert_status(NOT_CHANGE);
 
     packet_string = g_string_new_len(packet, packet_size);
-    packet_free();
     milter_command_encoder_encode_connect(encoder,
                                           &packet, &packet_size,
                                           host_name,
@@ -507,14 +511,16 @@ void
 test_helo (void)
 {
     const gchar fqdn[] = "delian";
+    const gchar *packet;
+    gsize packet_size;
 
     test_connect_with_macro();
-    packet_free();
     channel_free();
 
     reply_continue();
 
     cut_assert_true(milter_server_context_helo(context, fqdn));
+    pump_all_events();
     milter_test_assert_state(HELO);
     milter_test_assert_status(NOT_CHANGE);
 
@@ -528,14 +534,16 @@ void
 test_envelope_from (void)
 {
     const gchar from[] = "sender@example.com";
+    const gchar *packet;
+    gsize packet_size;
 
     test_helo();
-    packet_free();
     channel_free();
 
     reply_continue();
 
     cut_assert_true(milter_server_context_envelope_from(context, from));
+    pump_all_events();
     milter_test_assert_state(ENVELOPE_FROM);
     milter_test_assert_status(NOT_CHANGE);
 
@@ -550,14 +558,16 @@ void
 test_envelope_recipient (void)
 {
     const gchar recipient[] = "receiver@example.com";
+    const gchar *packet;
+    gsize packet_size;
 
     test_envelope_from();
-    packet_free();
     channel_free();
 
     reply_continue();
 
     cut_assert_true(milter_server_context_envelope_recipient(context, recipient));
+    pump_all_events();
     milter_test_assert_state(ENVELOPE_RECIPIENT);
     milter_test_assert_status(NOT_CHANGE);
 
@@ -572,13 +582,16 @@ test_envelope_recipient (void)
 void
 test_data (void)
 {
+    const gchar *packet;
+    gsize packet_size;
+
     test_envelope_recipient();
-    packet_free();
     channel_free();
 
     reply_continue();
 
     cut_assert_true(milter_server_context_data(context));
+    pump_all_events();
     milter_test_assert_state(DATA);
     milter_test_assert_status(NOT_CHANGE);
 
@@ -594,12 +607,12 @@ test_data_with_protocol_version2 (void)
     protocol_version = 2;
 
     test_envelope_recipient();
-    packet_free();
     channel_free();
 
     reply_continue();
 
     cut_assert_true(milter_server_context_data(context));
+    pump_all_events();
     milter_test_assert_state(DATA);
     milter_test_assert_status(NOT_CHANGE);
 
@@ -612,14 +625,16 @@ void
 test_unknown (void)
 {
     const gchar command[] = "Who am I?";
+    const gchar *packet;
+    gsize packet_size;
 
     test_data();
-    packet_free();
     channel_free();
 
     reply_continue();
 
     cut_assert_true(milter_server_context_unknown(context, command));
+    pump_all_events();
     milter_test_assert_state(UNKNOWN);
     milter_test_assert_status(NOT_CHANGE);
 
@@ -634,14 +649,16 @@ test_header (void)
 {
     const gchar name[] = "X-HEADER-NAME";
     const gchar value[] = "MilterServerContext test";
+    const gchar *packet;
+    gsize packet_size;
 
     test_data();
-    packet_free();
     channel_free();
 
     reply_continue();
 
     cut_assert_true(milter_server_context_header(context, name, value));
+    pump_all_events();
     milter_test_assert_state(HEADER);
     milter_test_assert_status(NOT_CHANGE);
 
@@ -656,13 +673,16 @@ test_header (void)
 void
 test_end_of_header (void)
 {
+    const gchar *packet;
+    gsize packet_size;
+
     test_header();
-    packet_free();
     channel_free();
 
     reply_continue();
 
     cut_assert_true(milter_server_context_end_of_header(context));
+    pump_all_events();
     milter_test_assert_state(END_OF_HEADER);
     milter_test_assert_status(NOT_CHANGE);
 
@@ -676,15 +696,17 @@ void
 test_body (void)
 {
     const gchar chunk[] = "This is a body text.";
+    const gchar *packet;
+    gsize packet_size;
     gsize packed_size;
 
     test_end_of_header();
-    packet_free();
     channel_free();
 
     reply_continue();
 
     cut_assert_true(milter_server_context_body(context, chunk, strlen(chunk)));
+    pump_all_events();
     milter_test_assert_state(BODY);
     milter_test_assert_status(NOT_CHANGE);
 
@@ -700,15 +722,17 @@ void
 test_end_of_message (void)
 {
     const gchar chunk[] = "This is a end_of_message text.";
+    const gchar *packet;
+    gsize packet_size;
 
     test_body();
-    packet_free();
     channel_free();
 
     reply_continue();
 
     cut_assert_true(milter_server_context_end_of_message(context,
                                                          chunk, strlen(chunk)));
+    pump_all_events();
     milter_test_assert_state(END_OF_MESSAGE);
     milter_test_assert_status(NOT_CHANGE);
 
@@ -722,13 +746,16 @@ test_end_of_message (void)
 void
 test_end_of_message_without_chunk (void)
 {
+    const gchar *packet;
+    gsize packet_size;
+
     test_body();
-    packet_free();
     channel_free();
 
     reply_continue();
 
     cut_assert_true(milter_server_context_end_of_message(context, NULL, 0));
+    pump_all_events();
     milter_test_assert_state(END_OF_MESSAGE);
     milter_test_assert_status(NOT_CHANGE);
 
@@ -742,13 +769,16 @@ test_end_of_message_without_chunk (void)
 void
 test_quit (void)
 {
+    const gchar *packet;
+    gsize packet_size;
+
     test_end_of_message_without_chunk();
-    packet_free();
     channel_free();
 
     reply_continue();
 
     cut_assert_true(milter_server_context_quit(context));
+    pump_all_events();
     milter_test_assert_state(QUIT);
     milter_test_assert_status(NOT_CHANGE);
 
@@ -772,13 +802,16 @@ test_quit (void)
 void
 test_quit_after_connect (void)
 {
+    const gchar *packet;
+    gsize packet_size;
+
     test_connect();
-    packet_free();
     channel_free();
 
     reply_continue();
 
     cut_assert_true(milter_server_context_quit(context));
+    pump_all_events();
     milter_test_assert_state(QUIT);
     milter_test_assert_status(NOT_CHANGE);
 
@@ -789,11 +822,14 @@ test_quit_after_connect (void)
 void
 test_abort (void)
 {
+    const gchar *packet;
+    gsize packet_size;
+
     test_body();
-    packet_free();
     channel_free();
 
     cut_assert_true(milter_server_context_abort(context));
+    pump_all_events();
     milter_test_assert_state(ABORT);
     milter_test_assert_status(ABORT);
 
@@ -804,21 +840,24 @@ test_abort (void)
 void
 test_abort_and_quit (void)
 {
+    const gchar *packet;
+    gsize packet_size;
+
     test_envelope_recipient();
-    packet_free();
     channel_free();
 
     cut_assert_true(milter_server_context_abort(context));
+    pump_all_events();
     milter_test_assert_state(ABORT);
     milter_test_assert_status(ABORT);
 
     cut_assert_true(milter_server_context_quit(context));
+    pump_all_events();
     milter_test_assert_state(QUIT);
     milter_test_assert_status(ABORT);
 
     milter_command_encoder_encode_abort(encoder, &packet, &packet_size);
     packet_string = g_string_new_len(packet, packet_size);
-    packet_free();
 
     milter_command_encoder_encode_quit(encoder, &packet, &packet_size);
     g_string_append_len(packet_string, packet, packet_size);
