@@ -1,6 +1,6 @@
 /* -*- Mode: C; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /*
- *  Copyright (C) 2008-2009  Kouhei Sutou <kou@clear-code.com>
+ *  Copyright (C) 2008-2011  Kouhei Sutou <kou@clear-code.com>
  *
  *  This library is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -55,6 +55,8 @@ void test_macro (gconstpointer data);
 void data_invalid_spec (void);
 void test_invalid_spec (gconstpointer data);
 
+static MilterEventLoop *loop;
+
 static MilterTestClient *client;
 static MilterDecoder *decoder;
 static MilterReplyEncoder *encoder;
@@ -63,12 +65,11 @@ static GArray *command;
 static gchar *server_path;
 static gchar *fixtures_path;
 
-static gsize packet_size;
-static gchar *packet;
 static GError *error;
 static gboolean reaped;
 static gint exit_status;
 static GString *output_string;
+static GString *normalized_output_string;
 static GString *error_string;
 static MilterStatus actual_status;
 
@@ -113,7 +114,8 @@ typedef struct _TestData
 void
 cut_startup (void)
 {
-    fixtures_path = g_build_filename(milter_test_get_base_dir(),
+    fixtures_path = g_build_filename(milter_test_get_source_dir(),
+                                     "test",
                                      "tool",
                                      "fixtures",
                                      NULL);
@@ -127,24 +129,17 @@ cut_shutdown (void)
 }
 
 static void
-packet_free (void)
-{
-    if (packet)
-        g_free(packet);
-    packet = NULL;
-}
-
-static void
-write_data (gchar *data, gsize data_size)
+write_data (const gchar *data, gsize data_size)
 {
     milter_test_client_write(client, data, data_size);
-    packet_free();
 }
 
 static void
 cb_negotiate (MilterCommandDecoder *decoder, MilterOption *option,
               TestData *test_data)
 {
+    const gchar *packet;
+    gsize packet_size;
     MilterOption *reply_option;
 
     reply_option = milter_option_new(6, test_data->actions, test_data->steps);
@@ -172,6 +167,8 @@ cb_define_macro (MilterDecoder *decoder, MilterCommand context,
 static void
 send_reply (MilterCommand command, TestData *test_data)
 {
+    const gchar *packet;
+    gsize packet_size;
     MilterReply reply;
     gint additional_flag;
     GList *reply_pair;
@@ -225,7 +222,6 @@ send_reply (MilterCommand command, TestData *test_data)
         milter_reply_encoder_encode_quarantine(encoder, &packet, &packet_size,
                                                "virus");
         write_data(packet, packet_size);
-        g_free(packet);
     case MILTER_REPLY_CONTINUE:
     default:
         milter_reply_encoder_encode_continue(encoder, &packet, &packet_size);
@@ -388,8 +384,10 @@ setup_server_command (void)
 }
 
 void
-setup (void)
+cut_setup (void)
 {
+    loop = milter_glib_event_loop_new(NULL);
+
     client = NULL;
     server = NULL;
     decoder = milter_command_decoder_new();
@@ -411,10 +409,10 @@ setup (void)
 
     setup_server_command();
 
-    packet = NULL;
     error = NULL;
     reaped = FALSE;
     output_string = g_string_new(NULL);
+    normalized_output_string = g_string_new(NULL);
     error_string = g_string_new(NULL);
 
     actual_status = MILTER_STATUS_DEFAULT;
@@ -433,7 +431,7 @@ setup (void)
 }
 
 void
-teardown (void)
+cut_teardown (void)
 {
     if (client)
         g_object_unref(client);
@@ -443,10 +441,11 @@ teardown (void)
         g_object_unref(encoder);
     if (command)
         g_array_free(command, TRUE);
-    packet_free();
 
     if (output_string)
         g_string_free(output_string, TRUE);
+    if (normalized_output_string)
+        g_string_free(normalized_output_string, TRUE);
     if (error_string)
         g_string_free(error_string, TRUE);
     if (error)
@@ -472,15 +471,6 @@ teardown (void)
         g_hash_table_unref(actual_defined_macros);
 }
 
-static void
-append_argument (GArray *array, const gchar *argument)
-{
-    gchar *dupped_argument;
-
-    dupped_argument = g_strdup(argument);
-    g_array_append_val(array, dupped_argument);
-}
-
 static void
 cb_output_received (GCutEgg *egg, const gchar *chunk, gsize size,
                     gpointer user_data)
@@ -502,10 +492,21 @@ cb_reaped (GCutEgg *egg, gint status, gpointer user_data)
     exit_status = status;
 }
 
+static void
+append_argument (const gchar *argument)
+{
+    gchar *dupped_argument;
+
+    dupped_argument = g_strdup(argument);
+    g_array_append_val(command, dupped_argument);
+}
+
 static void
 setup_server (const gchar *spec, const gchar *option_string)
 {
-    append_argument(command, "-s");
+    append_argument("--name");
+    append_argument("milter-test-server");
+    append_argument("-s");
     g_array_append_val(command, spec);
 
     if (option_string) {
@@ -533,7 +534,7 @@ setup_server (const gchar *spec, const gchar *option_string)
 static void
 setup_test_client (const gchar *spec, TestData *test_data)
 {
-    cut_trace(client = milter_test_client_new(spec, decoder));
+    cut_trace(client = milter_test_client_new(spec, decoder, loop));
     setup_command_signals(decoder, test_data);
 }
 
@@ -555,14 +556,20 @@ wait_for_reaping (gboolean check_success)
     gboolean timeout_waiting = TRUE;
     guint timeout_waiting_id;
 
-    timeout_waiting_id = g_timeout_add_seconds(1, cb_timeout_waiting,
-                                               &timeout_waiting);
+    timeout_waiting_id = milter_event_loop_add_timeout(loop, 1,
+                                                       cb_timeout_waiting,
+                                                       &timeout_waiting);
     while (timeout_waiting && !reaped) {
-        g_main_context_iteration(NULL, TRUE);
+        milter_event_loop_iterate(loop, TRUE);
     }
-    g_source_remove(timeout_waiting_id);
+    milter_event_loop_remove(loop, timeout_waiting_id);
     cut_assert_true(timeout_waiting, cut_message("timeout"));
 
+    g_string_assign(normalized_output_string,
+                    cut_take_replace(output_string->str,
+                                     "elapsed-time: [\\d.]+ seconds",
+                                     "elapsed-time: XXX seconds"));
+
     if (!check_success)
         return;
 
@@ -843,23 +850,6 @@ temporary_failure_data_result (void)
         NULL);
 }
 
-static void
-quarantine_data_result (void)
-{
-    ADD("body quarantine",
-        NULL,
-        MILTER_ACTION_NONE,
-        MILTER_STEP_NONE,
-        MILTER_STATUS_QUARANTINE,
-        create_replies(MILTER_COMMAND_BODY, MILTER_REPLY_QUARANTINE,
-                       NULL),
-        create_expected_command_receives(MILTER_COMMAND_BODY, 1,
-                                         MILTER_COMMAND_END_OF_MESSAGE, 1,
-                                         NULL),
-        NULL,
-        NULL);
-}
-
 static void
 reject_data_result (void)
 {
@@ -1181,7 +1171,6 @@ data_result (void)
     accept_data_result();
     discard_data_result();
     temporary_failure_data_result();
-    quarantine_data_result();
     reject_data_result();
     combined_data_result();
     reply_code_data_result();
@@ -1259,6 +1248,7 @@ milter_assert_equal_n_received (GHashTable *expected_command_receives)
 void
 test_result (gconstpointer data)
 {
+    GError *error = NULL;
     TestData *test_data = (TestData *)data;
 
     setup_test_client("inet:9999@localhost", test_data);
@@ -1322,7 +1312,7 @@ option_test_assert_connect_host (void)
 static void
 option_test_assert_connect_address (void)
 {
-    cut_assert_equal_string("mx.local.net", actual_connect_host);
+    cut_assert_equal_string("mx.example.net", actual_connect_host);
     cut_assert_equal_string("inet:12345@192.168.1.1", actual_connect_address);
 }
 
@@ -1342,16 +1332,18 @@ option_test_assert_helo_fqdn (void)
 static void
 option_test_assert_from (void)
 {
-    cut_assert_equal_string("from@example.com", actual_envelope_from);
+    cut_assert_equal_string("<from@example.com>", actual_envelope_from);
 }
 
 static void
 option_test_assert_recipients (void)
 {
-    cut_assert_equal_int(3, g_list_length(actual_recipients));
-    cut_assert_equal_string("recipient1@example.com", actual_recipients->data);
-    cut_assert_equal_string("recipient2@example.com", g_list_next(actual_recipients)->data);
-    cut_assert_equal_string("recipient3@example.com", g_list_last(actual_recipients)->data);
+    gcut_assert_equal_list_string(
+        gcut_take_new_list_string("<recipient1@example.com>",
+                                  "<recipient2@example.com>",
+                                  "<recipient3@example.com>",
+                                  NULL),
+        actual_recipients);
 }
 
 static void
@@ -1394,8 +1386,10 @@ option_test_assert_mail_file (void)
                                       "\n <mailto:milter-manager-commit-request@lists.sourceforge.net?subject=help>"};
 
     cut_assert_equal_string("<test-from@example.com>", actual_envelope_from);
-    cut_assert_equal_int(1, g_list_length(actual_recipients));
-    cut_assert_equal_string("<test-to@example.com>", actual_recipients->data);
+    gcut_assert_equal_list_string(
+        gcut_take_new_list_string("<test-to@example.com>",
+                                  NULL),
+        actual_recipients);
 
     cut_assert_equal_uint(10, milter_headers_length(actual_headers));
     milter_assert_equal_header(&expected_header1,
@@ -1423,29 +1417,30 @@ option_test_assert_mail_file (void)
                             actual_body_string->str);
 }
 
+#define RED_COLOR "\033[01;31m"
+#define GREEN_COLOR "\033[01;32m"
 #define NORMAL_COLOR "\033[00m"
 
 static void
 option_test_assert_output_message (void)
 {
-    const gchar *normalized_output;
-
-    normalized_output = cut_take_replace(output_string->str,
-                                         "elapsed-time: [\\d.]+ seconds",
-                                         "elapsed-time: XXX seconds");
     cut_assert_equal_string(
         "status: pass\n"
+        "elapsed-time: XXX seconds\n"
         "\n"
-        NORMAL_COLOR "  From: <kou+send@example.com>" NORMAL_COLOR "\n"
-        NORMAL_COLOR "  To: <kou+receive@example.com>" NORMAL_COLOR "\n"
-        "---------------------------------------\n"
+        "Envelope:------------------------------\n"
+        "  MAIL FROM:<sender@example.com>" "\n"
+        "  RCPT TO:<receiver@example.org>" "\n"
+        "Header:--------------------------------\n"
+        "  From: <sender@example.com>" "\n"
+        "  To: <receiver@example.org>" "\n"
+        "Body:----------------------------------\n"
         "La de da de da 1.\n"
         "La de da de da 2.\n"
         "La de da de da 3.\n"
-        "La de da de da 4.\n"
-        "\n"
-        "elapsed-time: XXX seconds\n",
-        normalized_output);
+        "La de da de da 4."
+        "\n",
+        normalized_output_string->str);
 }
 
 static void
@@ -1579,6 +1574,7 @@ data_option (void)
 void
 test_option (gconstpointer data)
 {
+    GError *error = NULL;
     OptionTestData *option_test_data = (OptionTestData*)data;
 
     setup_test_client("inet:9999@localhost", option_test_data->test_data);
@@ -1608,7 +1604,8 @@ typedef struct _EndOfMessageActionTestData
 
 static EndOfMessageActionTestData *
 end_of_message_action_test_data_new (EndOfMessageActionFunc action_func,
-                                     AssertFunc assert_func)
+                                     AssertFunc assert_func,
+                                     MilterStatus status)
 {
     EndOfMessageActionTestData *test_data;
 
@@ -1616,7 +1613,7 @@ end_of_message_action_test_data_new (EndOfMessageActionFunc action_func,
     test_data->test_data = result_test_data_new(NULL,
                                                 MILTER_ACTION_NONE,
                                                 MILTER_STEP_NONE,
-                                                MILTER_STATUS_NOT_CHANGE,
+                                                status,
                                                 NULL,
                                                 NULL,
                                                 NULL,
@@ -1636,10 +1633,11 @@ end_of_message_action_test_data_free (EndOfMessageActionTestData *data)
 static void
 change_from_function (void)
 {
-    packet_free();
+    const gchar *packet;
+    gsize packet_size;
 
     milter_reply_encoder_encode_change_from(encoder, &packet, &packet_size,
-                                            "changed-from@example.com", NULL);
+                                            "sender+changed@example.com", NULL);
 
     write_data(packet, packet_size);
 }
@@ -1647,19 +1645,35 @@ change_from_function (void)
 static void
 end_of_message_action_assert_change_from (void)
 {
-    cut_assert_match("(.*\n)*- From: <kou\\+send@example\\.com>"
-                     "(.*\n)*", output_string->str);
-    cut_assert_match("(.*\n)*\\+ From: changed-from@example\\.com"
-                     "(.*\n)*", output_string->str);
+    cut_assert_equal_string(
+        "status: pass\n"
+        "elapsed-time: XXX seconds\n"
+        "\n"
+        "Envelope:------------------------------\n"
+        RED_COLOR "- MAIL FROM:<sender@example.com>" NORMAL_COLOR "\n"
+        GREEN_COLOR "+ MAIL FROM:<sender+changed@example.com>" NORMAL_COLOR "\n"
+        "  RCPT TO:<receiver@example.org>" "\n"
+        "Header:--------------------------------\n"
+        "  From: <sender@example.com>" "\n"
+        "  To: <receiver@example.org>" "\n"
+        "Body:----------------------------------\n"
+        "La de da de da 1.\n"
+        "La de da de da 2.\n"
+        "La de da de da 3.\n"
+        "La de da de da 4."
+        "\n",
+        normalized_output_string->str);
 }
 
 static void
 add_recipient_function (void)
 {
-    packet_free();
+    const gchar *packet;
+    gsize packet_size;
 
-    milter_reply_encoder_encode_add_recipient(encoder, &packet, &packet_size, 
-                                              "added-to@example.com", NULL);
+    milter_reply_encoder_encode_add_recipient(encoder, &packet, &packet_size,
+                                              "receiver+added@example.org",
+                                              NULL);
 
     write_data(packet, packet_size);
 }
@@ -1667,19 +1681,34 @@ add_recipient_function (void)
 static void
 end_of_message_action_assert_add_recipient (void)
 {
-    cut_assert_match("(.*\n)*- To: <kou\\+receive@example\\.com>"
-                     "(.*\n)*", output_string->str);
-    cut_assert_match("(.*\n)*\\+ To: <kou\\+receive@example\\.com>, added-to@example\\.com"
-                     "(.*\n)*", output_string->str);
+    cut_assert_equal_string(
+        "status: pass\n"
+        "elapsed-time: XXX seconds\n"
+        "\n"
+        "Envelope:------------------------------\n"
+        "  MAIL FROM:<sender@example.com>" "\n"
+        "  RCPT TO:<receiver@example.org>" "\n"
+        GREEN_COLOR "+ RCPT TO:<receiver+added@example.org>" NORMAL_COLOR "\n"
+        "Header:--------------------------------\n"
+        "  From: <sender@example.com>" "\n"
+        "  To: <receiver@example.org>" "\n"
+        "Body:----------------------------------\n"
+        "La de da de da 1.\n"
+        "La de da de da 2.\n"
+        "La de da de da 3.\n"
+        "La de da de da 4."
+        "\n",
+        normalized_output_string->str);
 }
 
 static void
 delete_recipient_function (void)
 {
-    packet_free();
+    const gchar *packet;
+    gsize packet_size;
 
     milter_reply_encoder_encode_delete_recipient(encoder, &packet, &packet_size,
-                                                 "kou+receive@example.com");
+                                                 "receiver@example.org");
 
     write_data(packet, packet_size);
 }
@@ -1687,16 +1716,32 @@ delete_recipient_function (void)
 static void
 end_of_message_action_assert_delete_recipient (void)
 {
-    cut_assert_match("(.*\n)*- To: <kou\\+receive@example\\.com>"
-                     "(.*\n)*", output_string->str);
+    cut_assert_equal_string(
+        "status: pass\n"
+        "elapsed-time: XXX seconds\n"
+        "\n"
+        "Envelope:------------------------------\n"
+        "  MAIL FROM:<sender@example.com>" "\n"
+        RED_COLOR "- RCPT TO:<receiver@example.org>" NORMAL_COLOR "\n"
+        "Header:--------------------------------\n"
+        "  From: <sender@example.com>" "\n"
+        "  To: <receiver@example.org>" "\n"
+        "Body:----------------------------------\n"
+        "La de da de da 1.\n"
+        "La de da de da 2.\n"
+        "La de da de da 3.\n"
+        "La de da de da 4."
+        "\n",
+        normalized_output_string->str);
 }
 
 static void
 add_header_function (void)
 {
-    packet_free();
+    const gchar *packet;
+    gsize packet_size;
 
-    milter_reply_encoder_encode_add_header(encoder, &packet, &packet_size, 
+    milter_reply_encoder_encode_add_header(encoder, &packet, &packet_size,
                                            "X-Test-Header1", "TestHeader1Value");
 
     write_data(packet, packet_size);
@@ -1705,17 +1750,35 @@ add_header_function (void)
 static void
 end_of_message_action_assert_add_header (void)
 {
-    cut_assert_match("(.*\n)*\\+ X-Test-Header1: TestHeader1Value"
-                     "(.*\n)*", output_string->str);
+    cut_assert_equal_string(
+        "status: pass\n"
+        "elapsed-time: XXX seconds\n"
+        "\n"
+        "Envelope:------------------------------\n"
+        "  MAIL FROM:<sender@example.com>" "\n"
+        "  RCPT TO:<receiver@example.org>" "\n"
+        "Header:--------------------------------\n"
+        "  From: <sender@example.com>" "\n"
+        "  To: <receiver@example.org>" "\n"
+        GREEN_COLOR "+ X-Test-Header1: TestHeader1Value" NORMAL_COLOR "\n"
+        "Body:----------------------------------\n"
+        "La de da de da 1.\n"
+        "La de da de da 2.\n"
+        "La de da de da 3.\n"
+        "La de da de da 4."
+        "\n",
+        normalized_output_string->str);
 }
 
 static void
 insert_header_function (void)
 {
-    packet_free();
+    const gchar *packet;
+    gsize packet_size;
 
-    milter_reply_encoder_encode_insert_header(encoder, &packet, &packet_size, 
-                                              1, "X-Test-Header1", "TestHeader1Value");
+    milter_reply_encoder_encode_insert_header(encoder, &packet, &packet_size,
+                                              1, "To",
+                                              "<receiver+added@example.org>");
 
     write_data(packet, packet_size);
 }
@@ -1723,17 +1786,35 @@ insert_header_function (void)
 static void
 end_of_message_action_assert_insert_header (void)
 {
-    cut_assert_match("(.*\n)*\\+ X-Test-Header1: TestHeader1Value"
-                     "(.*\n)*", output_string->str);
+    cut_assert_equal_string(
+        "status: pass\n"
+        "elapsed-time: XXX seconds\n"
+        "\n"
+        "Envelope:------------------------------\n"
+        "  MAIL FROM:<sender@example.com>" "\n"
+        "  RCPT TO:<receiver@example.org>" "\n"
+        "Header:--------------------------------\n"
+        "  From: <sender@example.com>" "\n"
+        GREEN_COLOR "+ To: <receiver+added@example.org>" NORMAL_COLOR "\n"
+        "  To: <receiver@example.org>" "\n"
+        "Body:----------------------------------\n"
+        "La de da de da 1.\n"
+        "La de da de da 2.\n"
+        "La de da de da 3.\n"
+        "La de da de da 4."
+        "\n",
+        normalized_output_string->str);
 }
 
 static void
 change_header_function (void)
 {
-    packet_free();
+    const gchar *packet;
+    gsize packet_size;
 
     milter_reply_encoder_encode_change_header(encoder, &packet, &packet_size,
-                                              "To", 1, "changed-to@example.com");
+                                              "To", 1,
+                                              "<receiver+changed@example.org>");
 
     write_data(packet, packet_size);
 }
@@ -1741,16 +1822,31 @@ change_header_function (void)
 static void
 end_of_message_action_assert_change_header (void)
 {
-    cut_assert_match("(.*\n)*- To: <kou\\+receive@example\\.com>"
-                     "(.*\n)*", output_string->str);
-    cut_assert_match("(.*\n)*\\+ To: changed-to@example\\.com"
-                     "(.*\n)*", output_string->str);
+    cut_assert_equal_string(
+        "status: pass\n"
+        "elapsed-time: XXX seconds\n"
+        "\n"
+        "Envelope:------------------------------\n"
+        "  MAIL FROM:<sender@example.com>" "\n"
+        "  RCPT TO:<receiver@example.org>" "\n"
+        "Header:--------------------------------\n"
+        "  From: <sender@example.com>" "\n"
+        GREEN_COLOR "+ To: <receiver+changed@example.org>" NORMAL_COLOR "\n"
+        RED_COLOR "- To: <receiver@example.org>" NORMAL_COLOR "\n"
+        "Body:----------------------------------\n"
+        "La de da de da 1.\n"
+        "La de da de da 2.\n"
+        "La de da de da 3.\n"
+        "La de da de da 4."
+        "\n",
+        normalized_output_string->str);
 }
 
 static void
 delete_header_function (void)
 {
-    packet_free();
+    const gchar *packet;
+    gsize packet_size;
 
     milter_reply_encoder_encode_delete_header(encoder, &packet, &packet_size,
                                               "To", 1);
@@ -1761,18 +1857,33 @@ delete_header_function (void)
 static void
 end_of_message_action_assert_delete_header (void)
 {
-    cut_assert_match("(.*\n)*- To: <kou\\+receive@example\\.com>"
-                     "(.*\n)*", output_string->str);
+    cut_assert_equal_string(
+        "status: pass\n"
+        "elapsed-time: XXX seconds\n"
+        "\n"
+        "Envelope:------------------------------\n"
+        "  MAIL FROM:<sender@example.com>" "\n"
+        "  RCPT TO:<receiver@example.org>" "\n"
+        "Header:--------------------------------\n"
+        "  From: <sender@example.com>" "\n"
+        RED_COLOR "- To: <receiver@example.org>" NORMAL_COLOR "\n"
+        "Body:----------------------------------\n"
+        "La de da de da 1.\n"
+        "La de da de da 2.\n"
+        "La de da de da 3.\n"
+        "La de da de da 4."
+        "\n",
+        normalized_output_string->str);
 }
 
 static void
 replace_body_function (void)
 {
+    const gchar *packet;
+    gsize packet_size;
     const gchar replace_string[] = "This is the replaced message.";
     gsize packed_size;
 
-    packet_free();
-
     milter_reply_encoder_encode_replace_body(encoder, &packet, &packet_size,
                                              replace_string,
                                              strlen(replace_string),
@@ -1784,52 +1895,112 @@ replace_body_function (void)
 static void
 end_of_message_action_assert_replace_body (void)
 {
-    cut_assert_match("(.*\n)*-*"
-                     "This is the replaced message.",
-                     output_string->str);
+    cut_assert_equal_string(
+        "status: pass\n"
+        "elapsed-time: XXX seconds\n"
+        "\n"
+        "Envelope:------------------------------\n"
+        "  MAIL FROM:<sender@example.com>" "\n"
+        "  RCPT TO:<receiver@example.org>" "\n"
+        "Header:--------------------------------\n"
+        "  From: <sender@example.com>" "\n"
+        "  To: <receiver@example.org>" "\n"
+        "Body:----------------------------------\n"
+        "This is the replaced message.\n",
+        normalized_output_string->str);
+}
+
+static void
+quarantine_function (void)
+{
+    const gchar *packet;
+    gsize packet_size;
+    const gchar reason[] = "Virus is found.";
+
+    milter_reply_encoder_encode_quarantine(encoder, &packet, &packet_size,
+                                           reason);
+    write_data(packet, packet_size);
+}
+
+static void
+end_of_message_action_assert_quarantine (void)
+{
+    cut_assert_equal_string(
+        "status: quarantine\n"
+        "elapsed-time: XXX seconds\n"
+        "Quarantine reason: <Virus is found.>\n"
+        "\n"
+        "Envelope:------------------------------\n"
+        "  MAIL FROM:<sender@example.com>" "\n"
+        "  RCPT TO:<receiver@example.org>" "\n"
+        "Header:--------------------------------\n"
+        "  From: <sender@example.com>" "\n"
+        "  To: <receiver@example.org>" "\n"
+        "Body:----------------------------------\n"
+        "La de da de da 1.\n"
+        "La de da de da 2.\n"
+        "La de da de da 3.\n"
+        "La de da de da 4."
+        "\n",
+        normalized_output_string->str);
 }
 
 void
 data_end_of_message_action (void)
 {
-#define ADD(label, action_func, assert_func)                            \
+#define ADD(label, action_func, assert_func, status)                    \
     cut_add_data(label,                                                 \
                  end_of_message_action_test_data_new(action_func,       \
-                                                     assert_func),      \
+                                                     assert_func,       \
+                                                     status),           \
                  end_of_message_action_test_data_free,                  \
                  NULL)
 
     ADD("change-from",
         change_from_function,
-        end_of_message_action_assert_change_from);
+        end_of_message_action_assert_change_from,
+        MILTER_STATUS_NOT_CHANGE);
     ADD("add-recipient",
         add_recipient_function,
-        end_of_message_action_assert_add_recipient);
+        end_of_message_action_assert_add_recipient,
+        MILTER_STATUS_NOT_CHANGE);
     ADD("delete-recipient",
         delete_recipient_function,
-        end_of_message_action_assert_delete_recipient);
+        end_of_message_action_assert_delete_recipient,
+        MILTER_STATUS_NOT_CHANGE);
     ADD("add-header",
         add_header_function,
-        end_of_message_action_assert_add_header);
+        end_of_message_action_assert_add_header,
+        MILTER_STATUS_NOT_CHANGE);
     ADD("insert-header",
         insert_header_function,
-        end_of_message_action_assert_insert_header);
+        end_of_message_action_assert_insert_header,
+        MILTER_STATUS_NOT_CHANGE);
     ADD("change-header",
         change_header_function,
-        end_of_message_action_assert_change_header);
+        end_of_message_action_assert_change_header,
+        MILTER_STATUS_NOT_CHANGE);
     ADD("delete-header",
         delete_header_function,
-        end_of_message_action_assert_delete_header);
+        end_of_message_action_assert_delete_header,
+        MILTER_STATUS_NOT_CHANGE);
     ADD("replace-body",
         replace_body_function,
-        end_of_message_action_assert_replace_body);
+        end_of_message_action_assert_replace_body,
+        MILTER_STATUS_NOT_CHANGE);
+    ADD("quarantine",
+        quarantine_function,
+        end_of_message_action_assert_quarantine,
+        MILTER_STATUS_QUARANTINE);
 #undef ADD
 }
 
 void
 test_end_of_message_action (gconstpointer data)
 {
+    GError *error = NULL;
     EndOfMessageActionTestData *action_test_data;
+
     action_test_data = (EndOfMessageActionTestData*)data;
 
     setup_test_client("inet:9999@localhost", action_test_data->test_data);
@@ -1943,7 +2114,7 @@ create_expected_default_macros_table (void)
                         GINT_TO_POINTER(MILTER_COMMAND_ENVELOPE_RECIPIENT),
                         create_macro_hash_table("rcpt_mailer", "rcpt_mailer",
                                                 "rcpt_host", "rcpt_host",
-                                                "rcpt_addr", "<kou+receive@example.com>",
+                                                "rcpt_addr", "<receiver@example.org>",
                                                 NULL));
     g_hash_table_insert(expected_macros_table,
                         GINT_TO_POINTER(MILTER_COMMAND_END_OF_MESSAGE),
@@ -2043,8 +2214,8 @@ data_macro (void)
 void
 test_macro (gconstpointer data)
 {
-    MacroTestData *macro_test_data;
-    macro_test_data = (MacroTestData*)data;
+    GError *error = NULL;
+    MacroTestData *macro_test_data = (MacroTestData *)data;
 
     setup_test_client("inet:9999@localhost", macro_test_data->test_data);
     setup_server("inet:9999@localhost", NULL);
@@ -2113,7 +2284,8 @@ data_invalid_spec (void)
 void
 test_invalid_spec (gconstpointer data)
 {
-    InvalidSpecTestData *test_data = (InvalidSpecTestData*)data;
+    GError *error = NULL;
+    InvalidSpecTestData *test_data = (InvalidSpecTestData *)data;
 
     setup_server(test_data->spec, NULL);
     gcut_egg_hatch(server, &error);
